diff --git a/Directory.Build.props b/Directory.Build.props
index 21b8b402e0..2cd40c8675 100644
--- a/Directory.Build.props
+++ b/Directory.Build.props
@@ -16,9 +16,9 @@
-
+
-
+
$(MSBuildThisFileDirectory)CodeAnalysis\osu.ruleset
diff --git a/osu.Android.props b/osu.Android.props
index 336479c40a..a406cdf08a 100644
--- a/osu.Android.props
+++ b/osu.Android.props
@@ -52,6 +52,6 @@
-
+
diff --git a/osu.Android/OsuGameAndroid.cs b/osu.Android/OsuGameAndroid.cs
index 84f215f930..19ed7ffcf5 100644
--- a/osu.Android/OsuGameAndroid.cs
+++ b/osu.Android/OsuGameAndroid.cs
@@ -18,7 +18,8 @@ namespace osu.Android
try
{
- string versionName = packageInfo.VersionCode.ToString();
+ // todo: needs checking before play store redeploy.
+ string versionName = packageInfo.VersionName;
// undo play store version garbling
return new Version(int.Parse(versionName.Substring(0, 4)), int.Parse(versionName.Substring(4, 4)), int.Parse(versionName.Substring(8, 1)));
}
diff --git a/osu.Desktop/OsuGameDesktop.cs b/osu.Desktop/OsuGameDesktop.cs
index f05ee48914..9351e17419 100644
--- a/osu.Desktop/OsuGameDesktop.cs
+++ b/osu.Desktop/OsuGameDesktop.cs
@@ -6,15 +6,14 @@ using System.IO;
using System.Linq;
using System.Reflection;
using System.Threading.Tasks;
+using Microsoft.Win32;
using osu.Desktop.Overlays;
using osu.Framework.Platform;
using osu.Game;
using osuTK.Input;
-using Microsoft.Win32;
using osu.Desktop.Updater;
using osu.Framework;
using osu.Framework.Logging;
-using osu.Framework.Platform.Windows;
using osu.Framework.Screens;
using osu.Game.Screens.Menu;
using osu.Game.Updater;
@@ -37,7 +36,11 @@ namespace osu.Desktop
try
{
if (Host is DesktopGameHost desktopHost)
- return new StableStorage(desktopHost);
+ {
+ string stablePath = getStableInstallPath();
+ if (!string.IsNullOrEmpty(stablePath))
+ return new DesktopStorage(stablePath, desktopHost);
+ }
}
catch (Exception)
{
@@ -47,6 +50,35 @@ namespace osu.Desktop
return null;
}
+ private string getStableInstallPath()
+ {
+ static bool checkExists(string p) => Directory.Exists(Path.Combine(p, "Songs"));
+
+ string stableInstallPath;
+
+ try
+ {
+ using (RegistryKey key = Registry.ClassesRoot.OpenSubKey("osu"))
+ stableInstallPath = key?.OpenSubKey(@"shell\open\command")?.GetValue(string.Empty).ToString().Split('"')[1].Replace("osu!.exe", "");
+
+ if (checkExists(stableInstallPath))
+ return stableInstallPath;
+ }
+ catch
+ {
+ }
+
+ stableInstallPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), @"osu!");
+ if (checkExists(stableInstallPath))
+ return stableInstallPath;
+
+ stableInstallPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.UserProfile), ".osu");
+ if (checkExists(stableInstallPath))
+ return stableInstallPath;
+
+ return null;
+ }
+
protected override UpdateManager CreateUpdateManager()
{
switch (RuntimeInfo.OS)
@@ -111,45 +143,5 @@ namespace osu.Desktop
Task.Factory.StartNew(() => Import(filePaths), TaskCreationOptions.LongRunning);
}
-
- ///
- /// A method of accessing an osu-stable install in a controlled fashion.
- ///
- private class StableStorage : WindowsStorage
- {
- protected override string LocateBasePath()
- {
- static bool checkExists(string p) => Directory.Exists(Path.Combine(p, "Songs"));
-
- string stableInstallPath;
-
- try
- {
- using (RegistryKey key = Registry.ClassesRoot.OpenSubKey("osu"))
- stableInstallPath = key?.OpenSubKey(@"shell\open\command")?.GetValue(string.Empty).ToString().Split('"')[1].Replace("osu!.exe", "");
-
- if (checkExists(stableInstallPath))
- return stableInstallPath;
- }
- catch
- {
- }
-
- stableInstallPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), @"osu!");
- if (checkExists(stableInstallPath))
- return stableInstallPath;
-
- stableInstallPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.UserProfile), ".osu");
- if (checkExists(stableInstallPath))
- return stableInstallPath;
-
- return null;
- }
-
- public StableStorage(DesktopGameHost host)
- : base(string.Empty, host)
- {
- }
- }
}
}
diff --git a/osu.Desktop/Updater/SquirrelUpdateManager.cs b/osu.Desktop/Updater/SquirrelUpdateManager.cs
index 60b47a8b3a..ade8460dd7 100644
--- a/osu.Desktop/Updater/SquirrelUpdateManager.cs
+++ b/osu.Desktop/Updater/SquirrelUpdateManager.cs
@@ -43,7 +43,7 @@ namespace osu.Desktop.Updater
private async void checkForUpdateAsync(bool useDeltaPatching = true, UpdateProgressNotification notification = null)
{
- //should we schedule a retry on completion of this check?
+ // should we schedule a retry on completion of this check?
bool scheduleRecheck = true;
try
@@ -52,7 +52,7 @@ namespace osu.Desktop.Updater
var info = await updateManager.CheckForUpdate(!useDeltaPatching);
if (info.ReleasesToApply.Count == 0)
- //no updates available. bail and retry later.
+ // no updates available. bail and retry later.
return;
if (notification == null)
@@ -81,8 +81,8 @@ namespace osu.Desktop.Updater
{
logger.Add(@"delta patching failed; will attempt full download!");
- //could fail if deltas are unavailable for full update path (https://github.com/Squirrel/Squirrel.Windows/issues/959)
- //try again without deltas.
+ // could fail if deltas are unavailable for full update path (https://github.com/Squirrel/Squirrel.Windows/issues/959)
+ // try again without deltas.
checkForUpdateAsync(false, notification);
scheduleRecheck = false;
}
@@ -101,7 +101,7 @@ namespace osu.Desktop.Updater
{
if (scheduleRecheck)
{
- //check again in 30 minutes.
+ // check again in 30 minutes.
Scheduler.AddDelayed(() => checkForUpdateAsync(), 60000 * 30);
}
}
diff --git a/osu.Game.Rulesets.Catch.Tests/CatchDifficultyCalculatorTest.cs b/osu.Game.Rulesets.Catch.Tests/CatchDifficultyCalculatorTest.cs
index 51fe0b035d..ee416e5a38 100644
--- a/osu.Game.Rulesets.Catch.Tests/CatchDifficultyCalculatorTest.cs
+++ b/osu.Game.Rulesets.Catch.Tests/CatchDifficultyCalculatorTest.cs
@@ -13,7 +13,7 @@ namespace osu.Game.Rulesets.Catch.Tests
{
protected override string ResourceAssembly => "osu.Game.Rulesets.Catch";
- [TestCase(4.2058561036909863d, "diffcalc-test")]
+ [TestCase(4.050601681491468d, "diffcalc-test")]
public void Test(double expected, string name)
=> base.Test(expected, name);
diff --git a/osu.Game.Rulesets.Catch/Difficulty/CatchDifficultyCalculator.cs b/osu.Game.Rulesets.Catch/Difficulty/CatchDifficultyCalculator.cs
index d99325ff87..a317ef252d 100644
--- a/osu.Game.Rulesets.Catch/Difficulty/CatchDifficultyCalculator.cs
+++ b/osu.Game.Rulesets.Catch/Difficulty/CatchDifficultyCalculator.cs
@@ -19,7 +19,7 @@ namespace osu.Game.Rulesets.Catch.Difficulty
{
public class CatchDifficultyCalculator : DifficultyCalculator
{
- private const double star_scaling_factor = 0.145;
+ private const double star_scaling_factor = 0.153;
protected override int SectionLength => 750;
@@ -73,6 +73,9 @@ namespace osu.Game.Rulesets.Catch.Difficulty
{
halfCatcherWidth = Catcher.CalculateCatchWidth(beatmap.BeatmapInfo.BaseDifficulty) * 0.5f;
+ // For circle sizes above 5.5, reduce the catcher width further to simulate imperfect gameplay.
+ halfCatcherWidth *= 1 - (Math.Max(0, beatmap.BeatmapInfo.BaseDifficulty.CircleSize - 5.5f) * 0.0625f);
+
return new Skill[]
{
new Movement(halfCatcherWidth),
diff --git a/osu.Game.Rulesets.Catch/Difficulty/Skills/Movement.cs b/osu.Game.Rulesets.Catch/Difficulty/Skills/Movement.cs
index fd164907e0..5cd2f1f581 100644
--- a/osu.Game.Rulesets.Catch/Difficulty/Skills/Movement.cs
+++ b/osu.Game.Rulesets.Catch/Difficulty/Skills/Movement.cs
@@ -13,9 +13,9 @@ namespace osu.Game.Rulesets.Catch.Difficulty.Skills
{
private const float absolute_player_positioning_error = 16f;
private const float normalized_hitobject_radius = 41.0f;
- private const double direction_change_bonus = 12.5;
+ private const double direction_change_bonus = 21.0;
- protected override double SkillMultiplier => 850;
+ protected override double SkillMultiplier => 900;
protected override double StrainDecayBase => 0.2;
protected override double DecayWeight => 0.94;
@@ -24,6 +24,7 @@ namespace osu.Game.Rulesets.Catch.Difficulty.Skills
private float? lastPlayerPosition;
private float lastDistanceMoved;
+ private double lastStrainTime;
public Movement(float halfCatcherWidth)
{
@@ -45,47 +46,47 @@ namespace osu.Game.Rulesets.Catch.Difficulty.Skills
float distanceMoved = playerPosition - lastPlayerPosition.Value;
- double distanceAddition = Math.Pow(Math.Abs(distanceMoved), 1.3) / 500;
- double sqrtStrain = Math.Sqrt(catchCurrent.StrainTime);
+ double weightedStrainTime = catchCurrent.StrainTime + 13 + (3 / catchCurrent.ClockRate);
- double bonus = 0;
+ double distanceAddition = (Math.Pow(Math.Abs(distanceMoved), 1.3) / 510);
+ double sqrtStrain = Math.Sqrt(weightedStrainTime);
- // Direction changes give an extra point!
+ double edgeDashBonus = 0;
+
+ // Direction change bonus.
if (Math.Abs(distanceMoved) > 0.1)
{
if (Math.Abs(lastDistanceMoved) > 0.1 && Math.Sign(distanceMoved) != Math.Sign(lastDistanceMoved))
{
- double bonusFactor = Math.Min(absolute_player_positioning_error, Math.Abs(distanceMoved)) / absolute_player_positioning_error;
+ double bonusFactor = Math.Min(50, Math.Abs(distanceMoved)) / 50;
+ double antiflowFactor = Math.Max(Math.Min(70, Math.Abs(lastDistanceMoved)) / 70, 0.38);
- distanceAddition += direction_change_bonus / sqrtStrain * bonusFactor;
-
- // Bonus for tougher direction switches and "almost" hyperdashes at this point
- if (catchCurrent.LastObject.DistanceToHyperDash <= 10 / CatchPlayfield.BASE_WIDTH)
- bonus = 0.3 * bonusFactor;
+ distanceAddition += direction_change_bonus / Math.Sqrt(lastStrainTime + 16) * bonusFactor * antiflowFactor * Math.Max(1 - Math.Pow(weightedStrainTime / 1000, 3), 0);
}
// Base bonus for every movement, giving some weight to streams.
- distanceAddition += 7.5 * Math.Min(Math.Abs(distanceMoved), normalized_hitobject_radius * 2) / (normalized_hitobject_radius * 6) / sqrtStrain;
+ distanceAddition += 12.5 * Math.Min(Math.Abs(distanceMoved), normalized_hitobject_radius * 2) / (normalized_hitobject_radius * 6) / sqrtStrain;
}
- // Bonus for "almost" hyperdashes at corner points
- if (catchCurrent.LastObject.DistanceToHyperDash <= 10.0f / CatchPlayfield.BASE_WIDTH)
+ // Bonus for edge dashes.
+ if (catchCurrent.LastObject.DistanceToHyperDash <= 20.0f / CatchPlayfield.BASE_WIDTH)
{
if (!catchCurrent.LastObject.HyperDash)
- bonus += 1.0;
+ edgeDashBonus += 5.7;
else
{
// After a hyperdash we ARE in the correct position. Always!
playerPosition = catchCurrent.NormalizedPosition;
}
- distanceAddition *= 1.0 + bonus * ((10 - catchCurrent.LastObject.DistanceToHyperDash * CatchPlayfield.BASE_WIDTH) / 10);
+ distanceAddition *= 1.0 + edgeDashBonus * ((20 - catchCurrent.LastObject.DistanceToHyperDash * CatchPlayfield.BASE_WIDTH) / 20) * Math.Pow((Math.Min(catchCurrent.StrainTime * catchCurrent.ClockRate, 265) / 265), 1.5); // Edge Dashes are easier at lower ms values
}
lastPlayerPosition = playerPosition;
lastDistanceMoved = distanceMoved;
+ lastStrainTime = catchCurrent.StrainTime;
- return distanceAddition / catchCurrent.StrainTime;
+ return distanceAddition / weightedStrainTime;
}
}
}
diff --git a/osu.Game.Rulesets.Catch/Mods/CatchModRelax.cs b/osu.Game.Rulesets.Catch/Mods/CatchModRelax.cs
index 16414261a5..c1d24395e4 100644
--- a/osu.Game.Rulesets.Catch/Mods/CatchModRelax.cs
+++ b/osu.Game.Rulesets.Catch/Mods/CatchModRelax.cs
@@ -43,7 +43,7 @@ namespace osu.Game.Rulesets.Catch.Mods
RelativeSizeAxes = Axes.Both;
}
- //disable keyboard controls
+ // disable keyboard controls
public bool OnPressed(CatchAction action) => true;
public void OnReleased(CatchAction action)
diff --git a/osu.Game.Rulesets.Catch/Replays/CatchAutoGenerator.cs b/osu.Game.Rulesets.Catch/Replays/CatchAutoGenerator.cs
index b90b5812a6..7a33cb0577 100644
--- a/osu.Game.Rulesets.Catch/Replays/CatchAutoGenerator.cs
+++ b/osu.Game.Rulesets.Catch/Replays/CatchAutoGenerator.cs
@@ -55,7 +55,7 @@ namespace osu.Game.Rulesets.Catch.Replays
if (lastPosition - catcher_width_half < h.X && lastPosition + catcher_width_half > h.X)
{
- //we are already in the correct range.
+ // we are already in the correct range.
lastTime = h.StartTime;
addFrame(h.StartTime, lastPosition);
return;
@@ -72,14 +72,14 @@ namespace osu.Game.Rulesets.Catch.Replays
}
else if (dashRequired)
{
- //we do a movement in two parts - the dash part then the normal part...
+ // we do a movement in two parts - the dash part then the normal part...
double timeAtNormalSpeed = positionChange / movement_speed;
double timeWeNeedToSave = timeAtNormalSpeed - timeAvailable;
double timeAtDashSpeed = timeWeNeedToSave / 2;
float midPosition = (float)Interpolation.Lerp(lastPosition, h.X, (float)timeAtDashSpeed / timeAvailable);
- //dash movement
+ // dash movement
addFrame(h.StartTime - timeAvailable + 1, lastPosition, true);
addFrame(h.StartTime - timeAvailable + timeAtDashSpeed, midPosition);
addFrame(h.StartTime, h.X);
diff --git a/osu.Game.Rulesets.Osu/Beatmaps/OsuBeatmapProcessor.cs b/osu.Game.Rulesets.Osu/Beatmaps/OsuBeatmapProcessor.cs
index 3a829f72fa..f51f04bf87 100644
--- a/osu.Game.Rulesets.Osu/Beatmaps/OsuBeatmapProcessor.cs
+++ b/osu.Game.Rulesets.Osu/Beatmaps/OsuBeatmapProcessor.cs
@@ -66,7 +66,7 @@ namespace osu.Game.Rulesets.Osu.Beatmaps
double stackThreshold = objectN.TimePreempt * beatmap.BeatmapInfo.StackLeniency;
if (objectN.StartTime - endTime > stackThreshold)
- //We are no longer within stacking range of the next object.
+ // We are no longer within stacking range of the next object.
break;
if (Vector2Extensions.Distance(stackBaseObject.Position, objectN.Position) < stack_distance
@@ -88,7 +88,7 @@ namespace osu.Game.Rulesets.Osu.Beatmaps
}
}
- //Reverse pass for stack calculation.
+ // Reverse pass for stack calculation.
int extendedStartIndex = startIndex;
for (int i = extendedEndIndex; i > startIndex; i--)
@@ -124,7 +124,7 @@ namespace osu.Game.Rulesets.Osu.Beatmaps
double endTime = objectN.GetEndTime();
if (objectI.StartTime - endTime > stackThreshold)
- //We are no longer within stacking range of the previous object.
+ // We are no longer within stacking range of the previous object.
break;
// HitObjects before the specified update range haven't been reset yet
@@ -145,20 +145,20 @@ namespace osu.Game.Rulesets.Osu.Beatmaps
for (int j = n + 1; j <= i; j++)
{
- //For each object which was declared under this slider, we will offset it to appear *below* the slider end (rather than above).
+ // For each object which was declared under this slider, we will offset it to appear *below* the slider end (rather than above).
OsuHitObject objectJ = beatmap.HitObjects[j];
if (Vector2Extensions.Distance(objectN.EndPosition, objectJ.Position) < stack_distance)
objectJ.StackHeight -= offset;
}
- //We have hit a slider. We should restart calculation using this as the new base.
- //Breaking here will mean that the slider still has StackCount of 0, so will be handled in the i-outer-loop.
+ // We have hit a slider. We should restart calculation using this as the new base.
+ // Breaking here will mean that the slider still has StackCount of 0, so will be handled in the i-outer-loop.
break;
}
if (Vector2Extensions.Distance(objectN.Position, objectI.Position) < stack_distance)
{
- //Keep processing as if there are no sliders. If we come across a slider, this gets cancelled out.
+ // Keep processing as if there are no sliders. If we come across a slider, this gets cancelled out.
//NOTE: Sliders with start positions stacking are a special case that is also handled here.
objectN.StackHeight = objectI.StackHeight + 1;
@@ -177,7 +177,7 @@ namespace osu.Game.Rulesets.Osu.Beatmaps
if (objectN is Spinner) continue;
if (objectI.StartTime - objectN.StartTime > stackThreshold)
- //We are no longer within stacking range of the previous object.
+ // We are no longer within stacking range of the previous object.
break;
if (Vector2Extensions.Distance(objectN.EndPosition, objectI.Position) < stack_distance)
@@ -221,7 +221,7 @@ namespace osu.Game.Rulesets.Osu.Beatmaps
}
else if (Vector2Extensions.Distance(beatmap.HitObjects[j].Position, position2) < stack_distance)
{
- //Case for sliders - bump notes down and right, rather than up and left.
+ // Case for sliders - bump notes down and right, rather than up and left.
sliderStack++;
beatmap.HitObjects[j].StackHeight -= sliderStack;
startTime = beatmap.HitObjects[j].GetEndTime();
diff --git a/osu.Game.Rulesets.Osu/Mods/OsuModTransform.cs b/osu.Game.Rulesets.Osu/Mods/OsuModTransform.cs
index 44dba7715a..5e80d08667 100644
--- a/osu.Game.Rulesets.Osu/Mods/OsuModTransform.cs
+++ b/osu.Game.Rulesets.Osu/Mods/OsuModTransform.cs
@@ -49,7 +49,7 @@ namespace osu.Game.Rulesets.Osu.Mods
Vector2 originalPosition = drawable.Position;
Vector2 appearOffset = new Vector2(MathF.Cos(theta), MathF.Sin(theta)) * appearDistance;
- //the - 1 and + 1 prevents the hit objects to appear in the wrong position.
+ // the - 1 and + 1 prevents the hit objects to appear in the wrong position.
double appearTime = hitObject.StartTime - hitObject.TimePreempt - 1;
double moveDuration = hitObject.TimePreempt + 1;
diff --git a/osu.Game.Rulesets.Osu/Objects/Drawables/Pieces/MainCirclePiece.cs b/osu.Game.Rulesets.Osu/Objects/Drawables/Pieces/MainCirclePiece.cs
index e364c96426..cb3787a493 100644
--- a/osu.Game.Rulesets.Osu/Objects/Drawables/Pieces/MainCirclePiece.cs
+++ b/osu.Game.Rulesets.Osu/Objects/Drawables/Pieces/MainCirclePiece.cs
@@ -81,7 +81,7 @@ namespace osu.Game.Rulesets.Osu.Objects.Drawables.Pieces
using (BeginDelayedSequence(flash_in, true))
{
- //after the flash, we can hide some elements that were behind it
+ // after the flash, we can hide some elements that were behind it
ring.FadeOut();
circle.FadeOut();
number.FadeOut();
diff --git a/osu.Game.Tests/Beatmaps/IO/ImportBeatmapTest.cs b/osu.Game.Tests/Beatmaps/IO/ImportBeatmapTest.cs
index c6095ae404..ba6f5fc85c 100644
--- a/osu.Game.Tests/Beatmaps/IO/ImportBeatmapTest.cs
+++ b/osu.Game.Tests/Beatmaps/IO/ImportBeatmapTest.cs
@@ -31,7 +31,7 @@ namespace osu.Game.Tests.Beatmaps.IO
[Test]
public async Task TestImportWhenClosed()
{
- //unfortunately for the time being we need to reference osu.Framework.Desktop for a game host here.
+ // unfortunately for the time being we need to reference osu.Framework.Desktop for a game host here.
using (HeadlessGameHost host = new CleanRunHeadlessGameHost(nameof(TestImportWhenClosed)))
{
try
@@ -48,7 +48,7 @@ namespace osu.Game.Tests.Beatmaps.IO
[Test]
public async Task TestImportThenDelete()
{
- //unfortunately for the time being we need to reference osu.Framework.Desktop for a game host here.
+ // unfortunately for the time being we need to reference osu.Framework.Desktop for a game host here.
using (HeadlessGameHost host = new CleanRunHeadlessGameHost(nameof(TestImportThenDelete)))
{
try
@@ -69,7 +69,7 @@ namespace osu.Game.Tests.Beatmaps.IO
[Test]
public async Task TestImportThenImport()
{
- //unfortunately for the time being we need to reference osu.Framework.Desktop for a game host here.
+ // unfortunately for the time being we need to reference osu.Framework.Desktop for a game host here.
using (HeadlessGameHost host = new CleanRunHeadlessGameHost(nameof(TestImportThenImport)))
{
try
@@ -96,7 +96,7 @@ namespace osu.Game.Tests.Beatmaps.IO
[Test]
public async Task TestImportCorruptThenImport()
{
- //unfortunately for the time being we need to reference osu.Framework.Desktop for a game host here.
+ // unfortunately for the time being we need to reference osu.Framework.Desktop for a game host here.
using (HeadlessGameHost host = new CleanRunHeadlessGameHost(nameof(TestImportCorruptThenImport)))
{
try
@@ -138,7 +138,7 @@ namespace osu.Game.Tests.Beatmaps.IO
[Test]
public async Task TestRollbackOnFailure()
{
- //unfortunately for the time being we need to reference osu.Framework.Desktop for a game host here.
+ // unfortunately for the time being we need to reference osu.Framework.Desktop for a game host here.
using (HeadlessGameHost host = new CleanRunHeadlessGameHost(nameof(TestRollbackOnFailure)))
{
try
@@ -215,7 +215,7 @@ namespace osu.Game.Tests.Beatmaps.IO
[Test]
public async Task TestImportThenImportDifferentHash()
{
- //unfortunately for the time being we need to reference osu.Framework.Desktop for a game host here.
+ // unfortunately for the time being we need to reference osu.Framework.Desktop for a game host here.
using (HeadlessGameHost host = new CleanRunHeadlessGameHost(nameof(TestImportThenImportDifferentHash)))
{
try
@@ -246,7 +246,7 @@ namespace osu.Game.Tests.Beatmaps.IO
[Test]
public async Task TestImportThenDeleteThenImport()
{
- //unfortunately for the time being we need to reference osu.Framework.Desktop for a game host here.
+ // unfortunately for the time being we need to reference osu.Framework.Desktop for a game host here.
using (HeadlessGameHost host = new CleanRunHeadlessGameHost(nameof(TestImportThenDeleteThenImport)))
{
try
@@ -274,7 +274,7 @@ namespace osu.Game.Tests.Beatmaps.IO
[TestCase(false)]
public async Task TestImportThenDeleteThenImportWithOnlineIDMismatch(bool set)
{
- //unfortunately for the time being we need to reference osu.Framework.Desktop for a game host here.
+ // unfortunately for the time being we need to reference osu.Framework.Desktop for a game host here.
using (HeadlessGameHost host = new CleanRunHeadlessGameHost($"{nameof(TestImportThenDeleteThenImportWithOnlineIDMismatch)}-{set}"))
{
try
@@ -308,7 +308,7 @@ namespace osu.Game.Tests.Beatmaps.IO
[Test]
public async Task TestImportWithDuplicateBeatmapIDs()
{
- //unfortunately for the time being we need to reference osu.Framework.Desktop for a game host here.
+ // unfortunately for the time being we need to reference osu.Framework.Desktop for a game host here.
using (HeadlessGameHost host = new CleanRunHeadlessGameHost(nameof(TestImportWithDuplicateBeatmapIDs)))
{
try
@@ -695,12 +695,12 @@ namespace osu.Game.Tests.Beatmaps.IO
waitForOrAssert(() => (resultSets = store.QueryBeatmapSets(s => s.OnlineBeatmapSetID == 241526)).Any(),
@"BeatmapSet did not import to the database in allocated time.", timeout);
- //ensure we were stored to beatmap database backing...
+ // ensure we were stored to beatmap database backing...
Assert.IsTrue(resultSets.Count() == 1, $@"Incorrect result count found ({resultSets.Count()} but should be 1).");
IEnumerable queryBeatmaps() => store.QueryBeatmaps(s => s.BeatmapSet.OnlineBeatmapSetID == 241526 && s.BaseDifficultyID > 0);
IEnumerable queryBeatmapSets() => store.QueryBeatmapSets(s => s.OnlineBeatmapSetID == 241526);
- //if we don't re-check here, the set will be inserted but the beatmaps won't be present yet.
+ // if we don't re-check here, the set will be inserted but the beatmaps won't be present yet.
waitForOrAssert(() => queryBeatmaps().Count() == 12,
@"Beatmaps did not import to the database in allocated time", timeout);
waitForOrAssert(() => queryBeatmapSets().Count() == 1,
diff --git a/osu.Game.Tests/NonVisual/ControlPointInfoTest.cs b/osu.Game.Tests/NonVisual/ControlPointInfoTest.cs
index 158954106d..830e4bc603 100644
--- a/osu.Game.Tests/NonVisual/ControlPointInfoTest.cs
+++ b/osu.Game.Tests/NonVisual/ControlPointInfoTest.cs
@@ -35,7 +35,7 @@ namespace osu.Game.Tests.NonVisual
Assert.That(cpi.TimingPoints.Count, Is.EqualTo(2));
Assert.That(cpi.AllControlPoints.Count(), Is.EqualTo(2));
- cpi.Add(1000, new TimingControlPoint()); //is redundant
+ cpi.Add(1000, new TimingControlPoint()); // is redundant
Assert.That(cpi.Groups.Count, Is.EqualTo(2));
Assert.That(cpi.TimingPoints.Count, Is.EqualTo(2));
diff --git a/osu.Game.Tests/NonVisual/FramedReplayInputHandlerTest.cs b/osu.Game.Tests/NonVisual/FramedReplayInputHandlerTest.cs
index 7df7df22ea..92a60663de 100644
--- a/osu.Game.Tests/NonVisual/FramedReplayInputHandlerTest.cs
+++ b/osu.Game.Tests/NonVisual/FramedReplayInputHandlerTest.cs
@@ -46,12 +46,12 @@ namespace osu.Game.Tests.NonVisual
confirmCurrentFrame(0);
confirmNextFrame(1);
- //if we hit the first frame perfectly, time should progress to it.
+ // if we hit the first frame perfectly, time should progress to it.
setTime(1000, 1000);
confirmCurrentFrame(1);
confirmNextFrame(2);
- //in between non-important frames should progress based on input.
+ // in between non-important frames should progress based on input.
setTime(1200, 1200);
confirmCurrentFrame(1);
@@ -144,7 +144,7 @@ namespace osu.Game.Tests.NonVisual
confirmCurrentFrame(2);
confirmNextFrame(1);
- //ensure each frame plays out until start
+ // ensure each frame plays out until start
setTime(-500, 1000);
confirmCurrentFrame(1);
confirmNextFrame(0);
diff --git a/osu.Game.Tests/NonVisual/PeriodTrackerTest.cs b/osu.Game.Tests/NonVisual/PeriodTrackerTest.cs
new file mode 100644
index 0000000000..62c7732b66
--- /dev/null
+++ b/osu.Game.Tests/NonVisual/PeriodTrackerTest.cs
@@ -0,0 +1,85 @@
+// Copyright (c) ppy Pty Ltd . Licensed under the MIT Licence.
+// See the LICENCE file in the repository root for full licence text.
+
+using System;
+using System.Linq;
+using NUnit.Framework;
+using osu.Framework.Utils;
+using osu.Game.Utils;
+
+namespace osu.Game.Tests.NonVisual
+{
+ [TestFixture]
+ public class PeriodTrackerTest
+ {
+ private static readonly Period[] single_period = { new Period(1.0, 2.0) };
+
+ private static readonly Period[] unordered_periods =
+ {
+ new Period(-9.1, -8.3),
+ new Period(-3.4, 2.1),
+ new Period(9.0, 50.0),
+ new Period(5.25, 10.50)
+ };
+
+ [Test]
+ public void TestCheckValueInsideSinglePeriod()
+ {
+ var tracker = new PeriodTracker(single_period);
+
+ var period = single_period.Single();
+ Assert.IsTrue(tracker.IsInAny(period.Start));
+ Assert.IsTrue(tracker.IsInAny(getMidpoint(period)));
+ Assert.IsTrue(tracker.IsInAny(period.End));
+ }
+
+ [Test]
+ public void TestCheckValuesInsidePeriods()
+ {
+ var tracker = new PeriodTracker(unordered_periods);
+
+ foreach (var period in unordered_periods)
+ Assert.IsTrue(tracker.IsInAny(getMidpoint(period)));
+ }
+
+ [Test]
+ public void TestCheckValuesInRandomOrder()
+ {
+ var tracker = new PeriodTracker(unordered_periods);
+
+ foreach (var period in unordered_periods.OrderBy(_ => RNG.Next()))
+ Assert.IsTrue(tracker.IsInAny(getMidpoint(period)));
+ }
+
+ [Test]
+ public void TestCheckValuesOutOfPeriods()
+ {
+ var tracker = new PeriodTracker(new[]
+ {
+ new Period(1.0, 2.0),
+ new Period(3.0, 4.0)
+ });
+
+ Assert.IsFalse(tracker.IsInAny(0.9), "Time before first period is being considered inside");
+
+ Assert.IsFalse(tracker.IsInAny(2.1), "Time right after first period is being considered inside");
+ Assert.IsFalse(tracker.IsInAny(2.9), "Time right before second period is being considered inside");
+
+ Assert.IsFalse(tracker.IsInAny(4.1), "Time after last period is being considered inside");
+ }
+
+ [Test]
+ public void TestReversedPeriodHandling()
+ {
+ Assert.Throws(() =>
+ {
+ _ = new PeriodTracker(new[]
+ {
+ new Period(2.0, 1.0)
+ });
+ });
+ }
+
+ private double getMidpoint(Period period) => period.Start + (period.End - period.Start) / 2;
+ }
+}
diff --git a/osu.Game.Tests/Visual/Gameplay/TestSceneAutoplay.cs b/osu.Game.Tests/Visual/Gameplay/TestSceneAutoplay.cs
index 4b1c2ec256..0be949650e 100644
--- a/osu.Game.Tests/Visual/Gameplay/TestSceneAutoplay.cs
+++ b/osu.Game.Tests/Visual/Gameplay/TestSceneAutoplay.cs
@@ -52,7 +52,7 @@ namespace osu.Game.Tests.Visual.Gameplay
AddStep($"seek to break {breakIndex}", () => Player.GameplayClockContainer.Seek(destBreak().StartTime));
AddUntilStep("wait for seek to complete", () => Player.HUDOverlay.Progress.ReferenceClock.CurrentTime >= destBreak().StartTime);
- BreakPeriod destBreak() => Player.ChildrenOfType().First().Breaks.ElementAt(breakIndex);
+ BreakPeriod destBreak() => Beatmap.Value.Beatmap.Breaks.ElementAt(breakIndex);
}
}
}
diff --git a/osu.Game.Tests/Visual/Gameplay/TestSceneBreakTracker.cs b/osu.Game.Tests/Visual/Gameplay/TestSceneBreakTracker.cs
index 91d6f2f143..a6f996c30d 100644
--- a/osu.Game.Tests/Visual/Gameplay/TestSceneBreakTracker.cs
+++ b/osu.Game.Tests/Visual/Gameplay/TestSceneBreakTracker.cs
@@ -97,8 +97,6 @@ namespace osu.Game.Tests.Visual.Gameplay
loadBreaksStep("multiple breaks", testBreaks);
seekAndAssertBreak("seek to break start", testBreaks[1].StartTime, true);
- AddAssert("is skipped to break #2", () => breakTracker.CurrentBreakIndex == 1);
-
seekAndAssertBreak("seek to break middle", testBreaks[1].StartTime + testBreaks[1].Duration / 2, true);
seekAndAssertBreak("seek to break end", testBreaks[1].EndTime, false);
seekAndAssertBreak("seek to break after end", testBreaks[1].EndTime + 500, false);
@@ -174,8 +172,6 @@ namespace osu.Game.Tests.Visual.Gameplay
private readonly ManualClock manualClock;
private IFrameBasedClock originalClock;
- public new int CurrentBreakIndex => base.CurrentBreakIndex;
-
public double ManualClockTime
{
get => manualClock.CurrentTime;
diff --git a/osu.Game.Tests/Visual/SongSelect/TestScenePlaySongSelect.cs b/osu.Game.Tests/Visual/SongSelect/TestScenePlaySongSelect.cs
index aed8e19fb2..81fd1b66e5 100644
--- a/osu.Game.Tests/Visual/SongSelect/TestScenePlaySongSelect.cs
+++ b/osu.Game.Tests/Visual/SongSelect/TestScenePlaySongSelect.cs
@@ -24,10 +24,12 @@ using osu.Game.Rulesets.Mods;
using osu.Game.Rulesets.Osu;
using osu.Game.Rulesets.Osu.Mods;
using osu.Game.Rulesets.Taiko;
+using osu.Game.Scoring;
using osu.Game.Screens.Play;
using osu.Game.Screens.Select;
using osu.Game.Screens.Select.Carousel;
using osu.Game.Screens.Select.Filter;
+using osu.Game.Users;
using osuTK.Input;
namespace osu.Game.Tests.Visual.SongSelect
@@ -110,7 +112,7 @@ namespace osu.Game.Tests.Visual.SongSelect
createSongSelect();
- AddUntilStep("wait for initial selection", () => !Beatmap.IsDefault);
+ waitForInitialSelection();
WorkingBeatmap selected = null;
@@ -135,7 +137,7 @@ namespace osu.Game.Tests.Visual.SongSelect
createSongSelect();
- AddUntilStep("wait for initial selection", () => !Beatmap.IsDefault);
+ waitForInitialSelection();
WorkingBeatmap selected = null;
@@ -189,7 +191,7 @@ namespace osu.Game.Tests.Visual.SongSelect
createSongSelect();
- AddUntilStep("wait for initial selection", () => !Beatmap.IsDefault);
+ waitForInitialSelection();
WorkingBeatmap selected = null;
@@ -769,6 +771,76 @@ namespace osu.Game.Tests.Visual.SongSelect
AddAssert("Check first item in group selected", () => Beatmap.Value.BeatmapInfo == groupIcon.Items.First().Beatmap);
}
+ [Test]
+ public void TestChangeRulesetWhilePresentingScore()
+ {
+ BeatmapInfo getPresentBeatmap() => manager.QueryBeatmap(b => !b.BeatmapSet.DeletePending && b.RulesetID == 0);
+ BeatmapInfo getSwitchBeatmap() => manager.QueryBeatmap(b => !b.BeatmapSet.DeletePending && b.RulesetID == 1);
+
+ changeRuleset(0);
+
+ createSongSelect();
+
+ addRulesetImportStep(0);
+ addRulesetImportStep(1);
+
+ AddStep("present score", () =>
+ {
+ // this ruleset change should be overridden by the present.
+ Ruleset.Value = getSwitchBeatmap().Ruleset;
+
+ songSelect.PresentScore(new ScoreInfo
+ {
+ User = new User { Username = "woo" },
+ Beatmap = getPresentBeatmap(),
+ Ruleset = getPresentBeatmap().Ruleset
+ });
+ });
+
+ AddUntilStep("wait for results screen presented", () => !songSelect.IsCurrentScreen());
+
+ AddAssert("check beatmap is correct for score", () => Beatmap.Value.BeatmapInfo.Equals(getPresentBeatmap()));
+ AddAssert("check ruleset is correct for score", () => Ruleset.Value.ID == 0);
+ }
+
+ [Test]
+ public void TestChangeBeatmapWhilePresentingScore()
+ {
+ BeatmapInfo getPresentBeatmap() => manager.QueryBeatmap(b => !b.BeatmapSet.DeletePending && b.RulesetID == 0);
+ BeatmapInfo getSwitchBeatmap() => manager.QueryBeatmap(b => !b.BeatmapSet.DeletePending && b.RulesetID == 1);
+
+ changeRuleset(0);
+
+ addRulesetImportStep(0);
+ addRulesetImportStep(1);
+
+ createSongSelect();
+
+ AddStep("present score", () =>
+ {
+ // this beatmap change should be overridden by the present.
+ Beatmap.Value = manager.GetWorkingBeatmap(getSwitchBeatmap());
+
+ songSelect.PresentScore(new ScoreInfo
+ {
+ User = new User { Username = "woo" },
+ Beatmap = getPresentBeatmap(),
+ Ruleset = getPresentBeatmap().Ruleset
+ });
+ });
+
+ AddUntilStep("wait for results screen presented", () => !songSelect.IsCurrentScreen());
+
+ AddAssert("check beatmap is correct for score", () => Beatmap.Value.BeatmapInfo.Equals(getPresentBeatmap()));
+ AddAssert("check ruleset is correct for score", () => Ruleset.Value.ID == 0);
+ }
+
+ private void waitForInitialSelection()
+ {
+ AddUntilStep("wait for initial selection", () => !Beatmap.IsDefault);
+ AddUntilStep("wait for difficulty panels visible", () => songSelect.Carousel.ChildrenOfType().Any());
+ }
+
private int getBeatmapIndex(BeatmapSetInfo set, BeatmapInfo info) => set.Beatmaps.FindIndex(b => b == info);
private int getCurrentBeatmapIndex() => getBeatmapIndex(songSelect.Carousel.SelectedBeatmapSet, songSelect.Carousel.SelectedBeatmap);
@@ -876,6 +948,8 @@ namespace osu.Game.Tests.Visual.SongSelect
public WorkingBeatmap CurrentBeatmapDetailsBeatmap => BeatmapDetails.Beatmap;
public new BeatmapCarousel Carousel => base.Carousel;
+ public new void PresentScore(ScoreInfo score) => base.PresentScore(score);
+
protected override bool OnStart()
{
StartRequested?.Invoke();
diff --git a/osu.Game.Tournament/IPC/FileBasedIPC.cs b/osu.Game.Tournament/IPC/FileBasedIPC.cs
index eefa9fcfe6..53ba597a7e 100644
--- a/osu.Game.Tournament/IPC/FileBasedIPC.cs
+++ b/osu.Game.Tournament/IPC/FileBasedIPC.cs
@@ -8,7 +8,6 @@ using Microsoft.Win32;
using osu.Framework.Allocation;
using osu.Framework.Logging;
using osu.Framework.Platform;
-using osu.Framework.Platform.Windows;
using osu.Framework.Threading;
using osu.Game.Beatmaps;
using osu.Game.Beatmaps.Legacy;
@@ -52,7 +51,12 @@ namespace osu.Game.Tournament.IPC
try
{
- Storage = new StableStorage(host as DesktopGameHost);
+ var path = findStablePath();
+
+ if (string.IsNullOrEmpty(path))
+ return null;
+
+ Storage = new DesktopStorage(path, host as DesktopGameHost);
const string file_ipc_filename = "ipc.txt";
const string file_ipc_state_filename = "ipc-state.txt";
@@ -145,64 +149,50 @@ namespace osu.Game.Tournament.IPC
return Storage;
}
- ///
- /// A method of accessing an osu-stable install in a controlled fashion.
- ///
- private class StableStorage : WindowsStorage
+ private string findStablePath()
{
- protected override string LocateBasePath()
- {
- static bool checkExists(string p)
- {
- return File.Exists(Path.Combine(p, "ipc.txt"));
- }
+ static bool checkExists(string p) => File.Exists(Path.Combine(p, "ipc.txt"));
- string stableInstallPath = string.Empty;
+ string stableInstallPath = string.Empty;
+
+ try
+ {
+ try
+ {
+ stableInstallPath = Environment.GetEnvironmentVariable("OSU_STABLE_PATH");
+
+ if (checkExists(stableInstallPath))
+ return stableInstallPath;
+ }
+ catch
+ {
+ }
try
{
- try
- {
- stableInstallPath = Environment.GetEnvironmentVariable("OSU_STABLE_PATH");
+ using (RegistryKey key = Registry.ClassesRoot.OpenSubKey("osu"))
+ stableInstallPath = key?.OpenSubKey(@"shell\open\command")?.GetValue(string.Empty).ToString().Split('"')[1].Replace("osu!.exe", "");
- if (checkExists(stableInstallPath))
- return stableInstallPath;
- }
- catch
- {
- }
-
- try
- {
- using (RegistryKey key = Registry.ClassesRoot.OpenSubKey("osu"))
- stableInstallPath = key?.OpenSubKey(@"shell\open\command")?.GetValue(string.Empty).ToString().Split('"')[1].Replace("osu!.exe", "");
-
- if (checkExists(stableInstallPath))
- return stableInstallPath;
- }
- catch
- {
- }
-
- stableInstallPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), @"osu!");
if (checkExists(stableInstallPath))
return stableInstallPath;
-
- stableInstallPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.UserProfile), ".osu");
- if (checkExists(stableInstallPath))
- return stableInstallPath;
-
- return null;
}
- finally
+ catch
{
- Logger.Log($"Stable path for tourney usage: {stableInstallPath}");
}
- }
- public StableStorage(DesktopGameHost host)
- : base(string.Empty, host)
+ stableInstallPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData), @"osu!");
+ if (checkExists(stableInstallPath))
+ return stableInstallPath;
+
+ stableInstallPath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.UserProfile), ".osu");
+ if (checkExists(stableInstallPath))
+ return stableInstallPath;
+
+ return null;
+ }
+ finally
{
+ Logger.Log($"Stable path for tourney usage: {stableInstallPath}");
}
}
}
diff --git a/osu.Game.Tournament/Models/LadderInfo.cs b/osu.Game.Tournament/Models/LadderInfo.cs
index c2e6da9ca5..7794019437 100644
--- a/osu.Game.Tournament/Models/LadderInfo.cs
+++ b/osu.Game.Tournament/Models/LadderInfo.cs
@@ -32,5 +32,11 @@ namespace osu.Game.Tournament.Models
MinValue = 640,
MaxValue = 1366,
};
+
+ public Bindable PlayersPerTeam = new BindableInt(4)
+ {
+ MinValue = 3,
+ MaxValue = 4,
+ };
}
}
diff --git a/osu.Game.Tournament/Screens/Gameplay/GameplayScreen.cs b/osu.Game.Tournament/Screens/Gameplay/GameplayScreen.cs
index 64a5cd6dec..e4e3842369 100644
--- a/osu.Game.Tournament/Screens/Gameplay/GameplayScreen.cs
+++ b/osu.Game.Tournament/Screens/Gameplay/GameplayScreen.cs
@@ -36,7 +36,7 @@ namespace osu.Game.Tournament.Screens.Gameplay
[Resolved]
private TournamentMatchChatDisplay chat { get; set; }
- private Box chroma;
+ private Drawable chroma;
[BackgroundDependencyLoader]
private void load(LadderInfo ladder, MatchIPCInfo ipc, Storage storage)
@@ -61,16 +61,30 @@ namespace osu.Game.Tournament.Screens.Gameplay
Y = 110,
Anchor = Anchor.TopCentre,
Origin = Anchor.TopCentre,
- Children = new Drawable[]
+ Children = new[]
{
- chroma = new Box
+ chroma = new Container
{
- // chroma key area for stable gameplay
- Name = "chroma",
Anchor = Anchor.TopCentre,
Origin = Anchor.TopCentre,
Height = 512,
- Colour = new Color4(0, 255, 0, 255),
+ Children = new Drawable[]
+ {
+ new ChromaArea
+ {
+ Name = "Left chroma",
+ RelativeSizeAxes = Axes.Both,
+ Width = 0.5f,
+ },
+ new ChromaArea
+ {
+ Name = "Right chroma",
+ RelativeSizeAxes = Axes.Both,
+ Anchor = Anchor.TopRight,
+ Origin = Anchor.TopRight,
+ Width = 0.5f,
+ }
+ }
},
}
},
@@ -98,9 +112,15 @@ namespace osu.Game.Tournament.Screens.Gameplay
},
new SettingsSlider
{
- LabelText = "Chroma Width",
+ LabelText = "Chroma width",
Bindable = LadderInfo.ChromaKeyWidth,
KeyboardStep = 1,
+ },
+ new SettingsSlider
+ {
+ LabelText = "Players per team",
+ Bindable = LadderInfo.PlayersPerTeam,
+ KeyboardStep = 1,
}
}
}
@@ -201,5 +221,54 @@ namespace osu.Game.Tournament.Screens.Gameplay
lastState = state.NewValue;
}
}
+
+ private class ChromaArea : CompositeDrawable
+ {
+ [Resolved]
+ private LadderInfo ladder { get; set; }
+
+ [BackgroundDependencyLoader]
+ private void load()
+ {
+ // chroma key area for stable gameplay
+ Colour = new Color4(0, 255, 0, 255);
+
+ ladder.PlayersPerTeam.BindValueChanged(performLayout, true);
+ }
+
+ private void performLayout(ValueChangedEvent playerCount)
+ {
+ switch (playerCount.NewValue)
+ {
+ case 3:
+ InternalChildren = new Drawable[]
+ {
+ new Box
+ {
+ RelativeSizeAxes = Axes.Both,
+ Width = 0.5f,
+ Height = 0.5f,
+ Anchor = Anchor.TopCentre,
+ Origin = Anchor.TopCentre,
+ },
+ new Box
+ {
+ RelativeSizeAxes = Axes.Both,
+ Anchor = Anchor.BottomLeft,
+ Origin = Anchor.BottomLeft,
+ Height = 0.5f,
+ },
+ };
+ break;
+
+ default:
+ InternalChild = new Box
+ {
+ RelativeSizeAxes = Axes.Both,
+ };
+ break;
+ }
+ }
+ }
}
}
diff --git a/osu.Game/Beatmaps/BeatmapInfo.cs b/osu.Game/Beatmaps/BeatmapInfo.cs
index 90c100db05..3860f12baa 100644
--- a/osu.Game/Beatmaps/BeatmapInfo.cs
+++ b/osu.Game/Beatmaps/BeatmapInfo.cs
@@ -149,6 +149,11 @@ namespace osu.Game.Beatmaps
}
}
+ public string[] SearchableTerms => new[]
+ {
+ Version
+ }.Concat(Metadata?.SearchableTerms ?? Enumerable.Empty()).Where(s => !string.IsNullOrEmpty(s)).ToArray();
+
public override string ToString()
{
string version = string.IsNullOrEmpty(Version) ? string.Empty : $"[{Version}]";
diff --git a/osu.Game/Beatmaps/BeatmapManager.cs b/osu.Game/Beatmaps/BeatmapManager.cs
index b8dfac0342..34ad1df6bc 100644
--- a/osu.Game/Beatmaps/BeatmapManager.cs
+++ b/osu.Game/Beatmaps/BeatmapManager.cs
@@ -17,7 +17,6 @@ using osu.Framework.Graphics.Textures;
using osu.Framework.Lists;
using osu.Framework.Logging;
using osu.Framework.Platform;
-using osu.Framework.Threading;
using osu.Game.Beatmaps.Formats;
using osu.Game.Database;
using osu.Game.IO;
@@ -61,7 +60,7 @@ namespace osu.Game.Beatmaps
private readonly BeatmapStore beatmaps;
private readonly AudioManager audioManager;
private readonly GameHost host;
- private readonly BeatmapUpdateQueue updateQueue;
+ private readonly BeatmapOnlineLookupQueue onlineLookupQueue;
private readonly Storage exportStorage;
public BeatmapManager(Storage storage, IDatabaseContextFactory contextFactory, RulesetStore rulesets, IAPIProvider api, AudioManager audioManager, GameHost host = null,
@@ -78,7 +77,7 @@ namespace osu.Game.Beatmaps
beatmaps.BeatmapHidden += b => BeatmapHidden?.Invoke(b);
beatmaps.BeatmapRestored += b => BeatmapRestored?.Invoke(b);
- updateQueue = new BeatmapUpdateQueue(api);
+ onlineLookupQueue = new BeatmapOnlineLookupQueue(api, storage);
exportStorage = storage.GetStorageForDirectory("exports");
}
@@ -105,7 +104,7 @@ namespace osu.Game.Beatmaps
bool hadOnlineBeatmapIDs = beatmapSet.Beatmaps.Any(b => b.OnlineBeatmapID > 0);
- await updateQueue.UpdateAsync(beatmapSet, cancellationToken);
+ await onlineLookupQueue.UpdateAsync(beatmapSet, cancellationToken);
// ensure at least one beatmap was able to retrieve or keep an online ID, else drop the set ID.
if (hadOnlineBeatmapIDs && !beatmapSet.Beatmaps.Any(b => b.OnlineBeatmapID > 0))
@@ -141,7 +140,7 @@ namespace osu.Game.Beatmaps
{
var beatmapIds = beatmapSet.Beatmaps.Where(b => b.OnlineBeatmapID.HasValue).Select(b => b.OnlineBeatmapID).ToList();
- LogForModel(beatmapSet, "Validating online IDs...");
+ LogForModel(beatmapSet, $"Validating online IDs for {beatmapSet.Beatmaps.Count} beatmaps...");
// ensure all IDs are unique
if (beatmapIds.GroupBy(b => b).Any(g => g.Count() > 1))
@@ -382,7 +381,7 @@ namespace osu.Game.Beatmaps
foreach (var file in files.Where(f => f.Filename.EndsWith(".osu")))
{
using (var raw = Files.Store.GetStream(file.FileInfo.StoragePath))
- using (var ms = new MemoryStream()) //we need a memory stream so we can seek
+ using (var ms = new MemoryStream()) // we need a memory stream so we can seek
using (var sr = new LineBufferedReader(ms))
{
raw.CopyTo(ms);
@@ -446,71 +445,6 @@ namespace osu.Game.Beatmaps
protected override Texture GetBackground() => null;
protected override Track GetTrack() => null;
}
-
- private class BeatmapUpdateQueue
- {
- private readonly IAPIProvider api;
-
- private const int update_queue_request_concurrency = 4;
-
- private readonly ThreadedTaskScheduler updateScheduler = new ThreadedTaskScheduler(update_queue_request_concurrency, nameof(BeatmapUpdateQueue));
-
- public BeatmapUpdateQueue(IAPIProvider api)
- {
- this.api = api;
- }
-
- public Task UpdateAsync(BeatmapSetInfo beatmapSet, CancellationToken cancellationToken)
- {
- if (api?.State != APIState.Online)
- return Task.CompletedTask;
-
- LogForModel(beatmapSet, "Performing online lookups...");
- return Task.WhenAll(beatmapSet.Beatmaps.Select(b => UpdateAsync(beatmapSet, b, cancellationToken)).ToArray());
- }
-
- // todo: expose this when we need to do individual difficulty lookups.
- protected Task UpdateAsync(BeatmapSetInfo beatmapSet, BeatmapInfo beatmap, CancellationToken cancellationToken)
- => Task.Factory.StartNew(() => update(beatmapSet, beatmap), cancellationToken, TaskCreationOptions.HideScheduler, updateScheduler);
-
- private void update(BeatmapSetInfo set, BeatmapInfo beatmap)
- {
- if (api?.State != APIState.Online)
- return;
-
- var req = new GetBeatmapRequest(beatmap);
-
- req.Failure += fail;
-
- try
- {
- // intentionally blocking to limit web request concurrency
- api.Perform(req);
-
- var res = req.Result;
-
- if (res != null)
- {
- beatmap.Status = res.Status;
- beatmap.BeatmapSet.Status = res.BeatmapSet.Status;
- beatmap.BeatmapSet.OnlineBeatmapSetID = res.OnlineBeatmapSetID;
- beatmap.OnlineBeatmapID = res.OnlineBeatmapID;
-
- LogForModel(set, $"Online retrieval mapped {beatmap} to {res.OnlineBeatmapSetID} / {res.OnlineBeatmapID}.");
- }
- }
- catch (Exception e)
- {
- fail(e);
- }
-
- void fail(Exception e)
- {
- beatmap.OnlineBeatmapID = null;
- LogForModel(set, $"Online retrieval failed for {beatmap} ({e.Message})");
- }
- }
- }
}
///
diff --git a/osu.Game/Beatmaps/BeatmapManager_BeatmapOnlineLookupQueue.cs b/osu.Game/Beatmaps/BeatmapManager_BeatmapOnlineLookupQueue.cs
new file mode 100644
index 0000000000..2c79a664c5
--- /dev/null
+++ b/osu.Game/Beatmaps/BeatmapManager_BeatmapOnlineLookupQueue.cs
@@ -0,0 +1,195 @@
+// Copyright (c) ppy Pty Ltd . Licensed under the MIT Licence.
+// See the LICENCE file in the repository root for full licence text.
+
+using System;
+using System.Diagnostics.CodeAnalysis;
+using System.IO;
+using System.Linq;
+using System.Threading;
+using System.Threading.Tasks;
+using Dapper;
+using Microsoft.Data.Sqlite;
+using osu.Framework.Development;
+using osu.Framework.IO.Network;
+using osu.Framework.Logging;
+using osu.Framework.Platform;
+using osu.Framework.Threading;
+using osu.Game.Online.API;
+using osu.Game.Online.API.Requests;
+using SharpCompress.Compressors;
+using SharpCompress.Compressors.BZip2;
+
+namespace osu.Game.Beatmaps
+{
+ public partial class BeatmapManager
+ {
+ private class BeatmapOnlineLookupQueue
+ {
+ private readonly IAPIProvider api;
+ private readonly Storage storage;
+
+ private const int update_queue_request_concurrency = 4;
+
+ private readonly ThreadedTaskScheduler updateScheduler = new ThreadedTaskScheduler(update_queue_request_concurrency, nameof(BeatmapOnlineLookupQueue));
+
+ private FileWebRequest cacheDownloadRequest;
+
+ private const string cache_database_name = "online.db";
+
+ public BeatmapOnlineLookupQueue(IAPIProvider api, Storage storage)
+ {
+ this.api = api;
+ this.storage = storage;
+
+ // avoid downloading / using cache for unit tests.
+ if (!DebugUtils.IsNUnitRunning && !storage.Exists(cache_database_name))
+ prepareLocalCache();
+ }
+
+ public Task UpdateAsync(BeatmapSetInfo beatmapSet, CancellationToken cancellationToken)
+ {
+ if (api?.State != APIState.Online)
+ return Task.CompletedTask;
+
+ LogForModel(beatmapSet, "Performing online lookups...");
+ return Task.WhenAll(beatmapSet.Beatmaps.Select(b => UpdateAsync(beatmapSet, b, cancellationToken)).ToArray());
+ }
+
+ // todo: expose this when we need to do individual difficulty lookups.
+ protected Task UpdateAsync(BeatmapSetInfo beatmapSet, BeatmapInfo beatmap, CancellationToken cancellationToken)
+ => Task.Factory.StartNew(() => lookup(beatmapSet, beatmap), cancellationToken, TaskCreationOptions.HideScheduler, updateScheduler);
+
+ private void lookup(BeatmapSetInfo set, BeatmapInfo beatmap)
+ {
+ if (checkLocalCache(set, beatmap))
+ return;
+
+ if (api?.State != APIState.Online)
+ return;
+
+ var req = new GetBeatmapRequest(beatmap);
+
+ req.Failure += fail;
+
+ try
+ {
+ // intentionally blocking to limit web request concurrency
+ api.Perform(req);
+
+ var res = req.Result;
+
+ if (res != null)
+ {
+ beatmap.Status = res.Status;
+ beatmap.BeatmapSet.Status = res.BeatmapSet.Status;
+ beatmap.BeatmapSet.OnlineBeatmapSetID = res.OnlineBeatmapSetID;
+ beatmap.OnlineBeatmapID = res.OnlineBeatmapID;
+
+ LogForModel(set, $"Online retrieval mapped {beatmap} to {res.OnlineBeatmapSetID} / {res.OnlineBeatmapID}.");
+ }
+ }
+ catch (Exception e)
+ {
+ fail(e);
+ }
+
+ void fail(Exception e)
+ {
+ beatmap.OnlineBeatmapID = null;
+ LogForModel(set, $"Online retrieval failed for {beatmap} ({e.Message})");
+ }
+ }
+
+ private void prepareLocalCache()
+ {
+ string cacheFilePath = storage.GetFullPath(cache_database_name);
+ string compressedCacheFilePath = $"{cacheFilePath}.bz2";
+
+ cacheDownloadRequest = new FileWebRequest(compressedCacheFilePath, $"https://assets.ppy.sh/client-resources/{cache_database_name}.bz2");
+
+ cacheDownloadRequest.Failed += ex =>
+ {
+ File.Delete(compressedCacheFilePath);
+ File.Delete(cacheFilePath);
+
+ Logger.Log($"{nameof(BeatmapOnlineLookupQueue)}'s online cache download failed: {ex}", LoggingTarget.Database);
+ };
+
+ cacheDownloadRequest.Finished += () =>
+ {
+ try
+ {
+ using (var stream = File.OpenRead(cacheDownloadRequest.Filename))
+ using (var outStream = File.OpenWrite(cacheFilePath))
+ using (var bz2 = new BZip2Stream(stream, CompressionMode.Decompress, false))
+ bz2.CopyTo(outStream);
+
+ // set to null on completion to allow lookups to begin using the new source
+ cacheDownloadRequest = null;
+ }
+ catch (Exception ex)
+ {
+ Logger.Log($"{nameof(BeatmapOnlineLookupQueue)}'s online cache extraction failed: {ex}", LoggingTarget.Database);
+ File.Delete(cacheFilePath);
+ }
+ finally
+ {
+ File.Delete(compressedCacheFilePath);
+ }
+ };
+
+ cacheDownloadRequest.PerformAsync();
+ }
+
+ private bool checkLocalCache(BeatmapSetInfo set, BeatmapInfo beatmap)
+ {
+ // download is in progress (or was, and failed).
+ if (cacheDownloadRequest != null)
+ return false;
+
+ // database is unavailable.
+ if (!storage.Exists(cache_database_name))
+ return false;
+
+ try
+ {
+ using (var db = new SqliteConnection(storage.GetDatabaseConnectionString("online")))
+ {
+ var found = db.QuerySingleOrDefault(
+ "SELECT * FROM osu_beatmaps WHERE checksum = @MD5Hash OR beatmap_id = @OnlineBeatmapID OR filename = @Path", beatmap);
+
+ if (found != null)
+ {
+ var status = (BeatmapSetOnlineStatus)found.approved;
+
+ beatmap.Status = status;
+ beatmap.BeatmapSet.Status = status;
+ beatmap.BeatmapSet.OnlineBeatmapSetID = found.beatmapset_id;
+ beatmap.OnlineBeatmapID = found.beatmap_id;
+
+ LogForModel(set, $"Cached local retrieval for {beatmap}.");
+ return true;
+ }
+ }
+ }
+ catch (Exception ex)
+ {
+ LogForModel(set, $"Cached local retrieval for {beatmap} failed with {ex}.");
+ }
+
+ return false;
+ }
+
+ [Serializable]
+ [SuppressMessage("ReSharper", "InconsistentNaming")]
+ private class CachedOnlineBeatmapLookup
+ {
+ public int approved { get; set; }
+
+ public int? beatmapset_id { get; set; }
+
+ public int? beatmap_id { get; set; }
+ }
+ }
+ }
+}
diff --git a/osu.Game/Database/ArchiveModelManager.cs b/osu.Game/Database/ArchiveModelManager.cs
index 5e237d2ecb..839f9075e5 100644
--- a/osu.Game/Database/ArchiveModelManager.cs
+++ b/osu.Game/Database/ArchiveModelManager.cs
@@ -245,7 +245,7 @@ namespace osu.Game.Database
///
protected abstract string[] HashableFileTypes { get; }
- protected static void LogForModel(TModel model, string message, Exception e = null)
+ internal static void LogForModel(TModel model, string message, Exception e = null)
{
string prefix = $"[{(model?.Hash ?? "?????").Substring(0, 5)}]";
diff --git a/osu.Game/Graphics/Backgrounds/Triangles.cs b/osu.Game/Graphics/Backgrounds/Triangles.cs
index 590e4b2a5c..27027202ce 100644
--- a/osu.Game/Graphics/Backgrounds/Triangles.cs
+++ b/osu.Game/Graphics/Backgrounds/Triangles.cs
@@ -193,8 +193,8 @@ namespace osu.Game.Graphics.Backgrounds
float u1 = 1 - RNG.NextSingle(); //uniform(0,1] random floats
float u2 = 1 - RNG.NextSingle();
- float randStdNormal = (float)(Math.Sqrt(-2.0 * Math.Log(u1)) * Math.Sin(2.0 * Math.PI * u2)); //random normal(0,1)
- var scale = Math.Max(triangleScale * (mean + std_dev * randStdNormal), 0.1f); //random normal(mean,stdDev^2)
+ float randStdNormal = (float)(Math.Sqrt(-2.0 * Math.Log(u1)) * Math.Sin(2.0 * Math.PI * u2)); // random normal(0,1)
+ var scale = Math.Max(triangleScale * (mean + std_dev * randStdNormal), 0.1f); // random normal(mean,stdDev^2)
return new TriangleParticle { Scale = scale };
}
diff --git a/osu.Game/Graphics/Containers/OsuScrollContainer.cs b/osu.Game/Graphics/Containers/OsuScrollContainer.cs
index 1824fcd878..d504a11b22 100644
--- a/osu.Game/Graphics/Containers/OsuScrollContainer.cs
+++ b/osu.Game/Graphics/Containers/OsuScrollContainer.cs
@@ -158,7 +158,7 @@ namespace osu.Game.Graphics.Containers
{
if (!base.OnMouseDown(e)) return false;
- //note that we are changing the colour of the box here as to not interfere with the hover effect.
+ // note that we are changing the colour of the box here as to not interfere with the hover effect.
box.FadeColour(highlightColour, 100);
return true;
}
diff --git a/osu.Game/Graphics/DrawableDate.cs b/osu.Game/Graphics/DrawableDate.cs
index 8c520f4e10..8b6df4a834 100644
--- a/osu.Game/Graphics/DrawableDate.cs
+++ b/osu.Game/Graphics/DrawableDate.cs
@@ -139,7 +139,7 @@ namespace osu.Game.Graphics
return false;
dateText.Text = $"{date:d MMMM yyyy} ";
- timeText.Text = $"{date:hh:mm:ss \"UTC\"z}";
+ timeText.Text = $"{date:HH:mm:ss \"UTC\"z}";
return true;
}
diff --git a/osu.Game/IPC/ArchiveImportIPCChannel.cs b/osu.Game/IPC/ArchiveImportIPCChannel.cs
index 484db932f8..029908ec9d 100644
--- a/osu.Game/IPC/ArchiveImportIPCChannel.cs
+++ b/osu.Game/IPC/ArchiveImportIPCChannel.cs
@@ -32,7 +32,7 @@ namespace osu.Game.IPC
{
if (importer == null)
{
- //we want to contact a remote osu! to handle the import.
+ // we want to contact a remote osu! to handle the import.
await SendMessageAsync(new ArchiveImportMessage { Path = path });
return;
}
diff --git a/osu.Game/Online/API/APIAccess.cs b/osu.Game/Online/API/APIAccess.cs
index adfef1d11f..4945f7f185 100644
--- a/osu.Game/Online/API/APIAccess.cs
+++ b/osu.Game/Online/API/APIAccess.cs
@@ -127,7 +127,7 @@ namespace osu.Game.Online.API
case APIState.Offline:
case APIState.Connecting:
- //work to restore a connection...
+ // work to restore a connection...
if (!HasLogin)
{
State = APIState.Offline;
@@ -180,7 +180,7 @@ namespace osu.Game.Online.API
break;
}
- //hard bail if we can't get a valid access token.
+ // hard bail if we can't get a valid access token.
if (authentication.RequestAccessToken() == null)
{
Logout();
@@ -274,7 +274,7 @@ namespace osu.Game.Online.API
{
req.Perform(this);
- //we could still be in initialisation, at which point we don't want to say we're Online yet.
+ // we could still be in initialisation, at which point we don't want to say we're Online yet.
if (IsLoggedIn) State = APIState.Online;
failureCount = 0;
@@ -339,7 +339,7 @@ namespace osu.Game.Online.API
log.Add($@"API failure count is now {failureCount}");
if (failureCount < 3)
- //we might try again at an api level.
+ // we might try again at an api level.
return false;
if (State == APIState.Online)
diff --git a/osu.Game/Online/API/APIRequest.cs b/osu.Game/Online/API/APIRequest.cs
index 0bba04cac3..0f8acbb7af 100644
--- a/osu.Game/Online/API/APIRequest.cs
+++ b/osu.Game/Online/API/APIRequest.cs
@@ -98,7 +98,7 @@ namespace osu.Game.Online.API
if (checkAndScheduleFailure())
return;
- if (!WebRequest.Aborted) //could have been aborted by a Cancel() call
+ if (!WebRequest.Aborted) // could have been aborted by a Cancel() call
{
Logger.Log($@"Performing request {this}", LoggingTarget.Network);
WebRequest.Perform();
diff --git a/osu.Game/Online/Chat/Channel.cs b/osu.Game/Online/Chat/Channel.cs
index 6f67a95f53..dbb2da5c03 100644
--- a/osu.Game/Online/Chat/Channel.cs
+++ b/osu.Game/Online/Chat/Channel.cs
@@ -61,7 +61,7 @@ namespace osu.Game.Online.Chat
///
public event Action MessageRemoved;
- public bool ReadOnly => false; //todo not yet used.
+ public bool ReadOnly => false; // todo: not yet used.
public override string ToString() => Name;
diff --git a/osu.Game/Online/Chat/MessageFormatter.cs b/osu.Game/Online/Chat/MessageFormatter.cs
index 717de18c14..6af2561c89 100644
--- a/osu.Game/Online/Chat/MessageFormatter.cs
+++ b/osu.Game/Online/Chat/MessageFormatter.cs
@@ -78,13 +78,13 @@ namespace osu.Game.Online.Chat
{
result.Text = result.Text.Remove(index, m.Length).Insert(index, displayText);
- //since we just changed the line display text, offset any already processed links.
+ // since we just changed the line display text, offset any already processed links.
result.Links.ForEach(l => l.Index -= l.Index > index ? m.Length - displayText.Length : 0);
var details = GetLinkDetails(linkText);
result.Links.Add(new Link(linkText, index, displayText.Length, linkActionOverride ?? details.Action, details.Argument));
- //adjust the offset for processing the current matches group.
+ // adjust the offset for processing the current matches group.
captureOffset += m.Length - displayText.Length;
}
}
diff --git a/osu.Game/OsuGame.cs b/osu.Game/OsuGame.cs
index 8e62819c95..fdc8d94352 100644
--- a/osu.Game/OsuGame.cs
+++ b/osu.Game/OsuGame.cs
@@ -609,7 +609,7 @@ namespace osu.Game
loadComponentSingleFile(screenshotManager, Add);
- //overlay elements
+ // overlay elements
loadComponentSingleFile(beatmapListing = new BeatmapListingOverlay(), overlayContent.Add, true);
loadComponentSingleFile(dashboard = new DashboardOverlay(), overlayContent.Add, true);
var rankingsOverlay = loadComponentSingleFile(new RankingsOverlay(), overlayContent.Add, true);
@@ -781,7 +781,7 @@ namespace osu.Game
{
var previousLoadStream = asyncLoadStream;
- //chain with existing load stream
+ // chain with existing load stream
asyncLoadStream = Task.Run(async () =>
{
if (previousLoadStream != null)
diff --git a/osu.Game/Overlays/BeatmapListing/Panels/GridBeatmapPanel.cs b/osu.Game/Overlays/BeatmapListing/Panels/GridBeatmapPanel.cs
index 84d35da096..28c36e6c56 100644
--- a/osu.Game/Overlays/BeatmapListing/Panels/GridBeatmapPanel.cs
+++ b/osu.Game/Overlays/BeatmapListing/Panels/GridBeatmapPanel.cs
@@ -35,7 +35,7 @@ namespace osu.Game.Overlays.BeatmapListing.Panels
: base(beatmap)
{
Width = 380;
- Height = 140 + vertical_padding; //full height of all the elements plus vertical padding (autosize uses the image)
+ Height = 140 + vertical_padding; // full height of all the elements plus vertical padding (autosize uses the image)
}
protected override void LoadComplete()
diff --git a/osu.Game/Overlays/BeatmapSet/Header.cs b/osu.Game/Overlays/BeatmapSet/Header.cs
index 17fa689cd2..1ff08aab2c 100644
--- a/osu.Game/Overlays/BeatmapSet/Header.cs
+++ b/osu.Game/Overlays/BeatmapSet/Header.cs
@@ -140,7 +140,7 @@ namespace osu.Game.Overlays.BeatmapSet
{
Anchor = Anchor.BottomLeft,
Origin = Anchor.BottomLeft,
- Margin = new MarginPadding { Left = 3, Bottom = 4 }, //To better lineup with the font
+ Margin = new MarginPadding { Left = 3, Bottom = 4 }, // To better lineup with the font
},
}
},
diff --git a/osu.Game/Overlays/ChatOverlay.cs b/osu.Game/Overlays/ChatOverlay.cs
index 34afc3c431..5ba55f6d45 100644
--- a/osu.Game/Overlays/ChatOverlay.cs
+++ b/osu.Game/Overlays/ChatOverlay.cs
@@ -358,7 +358,7 @@ namespace osu.Game.Overlays
protected override void OnFocus(FocusEvent e)
{
- //this is necessary as textbox is masked away and therefore can't get focus :(
+ // this is necessary as textbox is masked away and therefore can't get focus :(
textbox.TakeFocus();
base.OnFocus(e);
}
diff --git a/osu.Game/Overlays/DialogOverlay.cs b/osu.Game/Overlays/DialogOverlay.cs
index 59d748bc5d..9f9dbdbaf1 100644
--- a/osu.Game/Overlays/DialogOverlay.cs
+++ b/osu.Game/Overlays/DialogOverlay.cs
@@ -50,7 +50,7 @@ namespace osu.Game.Overlays
{
if (v != Visibility.Hidden) return;
- //handle the dialog being dismissed.
+ // handle the dialog being dismissed.
dialog.Delay(PopupDialog.EXIT_DURATION).Expire();
if (dialog == CurrentDialog)
diff --git a/osu.Game/Overlays/Music/PlaylistItem.cs b/osu.Game/Overlays/Music/PlaylistItem.cs
index de2f916946..840fa51b4f 100644
--- a/osu.Game/Overlays/Music/PlaylistItem.cs
+++ b/osu.Game/Overlays/Music/PlaylistItem.cs
@@ -78,7 +78,7 @@ namespace osu.Game.Overlays.Music
{
text.Clear();
- //space after the title to put a space between the title and artist
+ // space after the title to put a space between the title and artist
titleSprites = text.AddText(title.Value + @" ", sprite => sprite.Font = OsuFont.GetFont(weight: FontWeight.Regular)).OfType();
text.AddText(artist.Value, sprite =>
diff --git a/osu.Game/Overlays/MusicController.cs b/osu.Game/Overlays/MusicController.cs
index c872f82b32..ded641b262 100644
--- a/osu.Game/Overlays/MusicController.cs
+++ b/osu.Game/Overlays/MusicController.cs
@@ -250,7 +250,7 @@ namespace osu.Game.Overlays
}
else
{
- //figure out the best direction based on order in playlist.
+ // figure out the best direction based on order in playlist.
var last = BeatmapSets.TakeWhile(b => b.ID != current.BeatmapSetInfo?.ID).Count();
var next = beatmap.NewValue == null ? -1 : BeatmapSets.TakeWhile(b => b.ID != beatmap.NewValue.BeatmapSetInfo?.ID).Count();
diff --git a/osu.Game/Overlays/News/NewsArticleCover.cs b/osu.Game/Overlays/News/NewsArticleCover.cs
index cca0cfb4a0..e3f5a8cea3 100644
--- a/osu.Game/Overlays/News/NewsArticleCover.cs
+++ b/osu.Game/Overlays/News/NewsArticleCover.cs
@@ -162,7 +162,7 @@ namespace osu.Game.Overlays.News
public string TooltipText => date.ToString("dddd dd MMMM yyyy hh:mm:ss UTCz").ToUpper();
}
- //fake API data struct to use for now as a skeleton for data, as there is no API struct for news article info for now
+ // fake API data struct to use for now as a skeleton for data, as there is no API struct for news article info for now
public class ArticleInfo
{
public string Title { get; set; }
diff --git a/osu.Game/Overlays/Notifications/ProgressNotification.cs b/osu.Game/Overlays/Notifications/ProgressNotification.cs
index 99836705c4..3105ecd742 100644
--- a/osu.Game/Overlays/Notifications/ProgressNotification.cs
+++ b/osu.Game/Overlays/Notifications/ProgressNotification.cs
@@ -39,7 +39,7 @@ namespace osu.Game.Overlays.Notifications
{
base.LoadComplete();
- //we may have received changes before we were displayed.
+ // we may have received changes before we were displayed.
updateState();
}
diff --git a/osu.Game/Overlays/NowPlayingOverlay.cs b/osu.Game/Overlays/NowPlayingOverlay.cs
index 118cb037cb..ebb4a96d14 100644
--- a/osu.Game/Overlays/NowPlayingOverlay.cs
+++ b/osu.Game/Overlays/NowPlayingOverlay.cs
@@ -261,7 +261,7 @@ namespace osu.Game.Overlays
// todo: this can likely be replaced with WorkingBeatmap.GetBeatmapAsync()
Task.Run(() =>
{
- if (beatmap?.Beatmap == null) //this is not needed if a placeholder exists
+ if (beatmap?.Beatmap == null) // this is not needed if a placeholder exists
{
title.Text = @"Nothing to play";
artist.Text = @"Nothing to play";
diff --git a/osu.Game/Overlays/OSD/Toast.cs b/osu.Game/Overlays/OSD/Toast.cs
index 5d36cac20e..1497ca8fa8 100644
--- a/osu.Game/Overlays/OSD/Toast.cs
+++ b/osu.Game/Overlays/OSD/Toast.cs
@@ -31,7 +31,7 @@ namespace osu.Game.Overlays.OSD
InternalChildren = new Drawable[]
{
- new Container //this container exists just to set a minimum width for the toast
+ new Container // this container exists just to set a minimum width for the toast
{
Anchor = Anchor.Centre,
Origin = Anchor.Centre,
diff --git a/osu.Game/Overlays/Profile/Header/Components/OverlinedInfoContainer.cs b/osu.Game/Overlays/Profile/Header/Components/OverlinedInfoContainer.cs
index b11e41f90f..9f56a34aa6 100644
--- a/osu.Game/Overlays/Profile/Header/Components/OverlinedInfoContainer.cs
+++ b/osu.Game/Overlays/Profile/Header/Components/OverlinedInfoContainer.cs
@@ -54,7 +54,7 @@ namespace osu.Game.Overlays.Profile.Header.Components
{
Font = OsuFont.GetFont(size: big ? 40 : 18, weight: FontWeight.Light)
},
- new Container //Add a minimum size to the FillFlowContainer
+ new Container // Add a minimum size to the FillFlowContainer
{
Width = minimumWidth,
}
diff --git a/osu.Game/Overlays/Profile/Header/MedalHeaderContainer.cs b/osu.Game/Overlays/Profile/Header/MedalHeaderContainer.cs
index a5938a3fe7..e7df4eb5eb 100644
--- a/osu.Game/Overlays/Profile/Header/MedalHeaderContainer.cs
+++ b/osu.Game/Overlays/Profile/Header/MedalHeaderContainer.cs
@@ -35,7 +35,7 @@ namespace osu.Game.Overlays.Profile.Header
RelativeSizeAxes = Axes.Both,
Colour = colourProvider.Background5,
},
- new Container //artificial shadow
+ new Container // artificial shadow
{
RelativeSizeAxes = Axes.X,
Height = 3,
diff --git a/osu.Game/Overlays/SearchableList/DisplayStyleControl.cs b/osu.Game/Overlays/SearchableList/DisplayStyleControl.cs
index a33f4eb30d..5ecb477a2f 100644
--- a/osu.Game/Overlays/SearchableList/DisplayStyleControl.cs
+++ b/osu.Game/Overlays/SearchableList/DisplayStyleControl.cs
@@ -91,6 +91,8 @@ namespace osu.Game.Overlays.SearchableList
protected override void Dispose(bool isDisposing)
{
+ base.Dispose(isDisposing);
+
bindable.ValueChanged -= Bindable_ValueChanged;
}
}
diff --git a/osu.Game/Overlays/SearchableList/SearchableListFilterControl.cs b/osu.Game/Overlays/SearchableList/SearchableListFilterControl.cs
index 117f905de4..d31470e685 100644
--- a/osu.Game/Overlays/SearchableList/SearchableListFilterControl.cs
+++ b/osu.Game/Overlays/SearchableList/SearchableListFilterControl.cs
@@ -94,7 +94,7 @@ namespace osu.Game.Overlays.SearchableList
RelativeSizeAxes = Axes.X,
},
},
- new Box //keep the tab strip part of autosize, but don't put it in the flow container
+ new Box // keep the tab strip part of autosize, but don't put it in the flow container
{
RelativeSizeAxes = Axes.X,
Height = 1,
diff --git a/osu.Game/Overlays/Toolbar/ToolbarButton.cs b/osu.Game/Overlays/Toolbar/ToolbarButton.cs
index d6b810366d..3d66d3c28e 100644
--- a/osu.Game/Overlays/Toolbar/ToolbarButton.cs
+++ b/osu.Game/Overlays/Toolbar/ToolbarButton.cs
@@ -110,7 +110,7 @@ namespace osu.Game.Overlays.Toolbar
tooltipContainer = new FillFlowContainer
{
Direction = FillDirection.Vertical,
- RelativeSizeAxes = Axes.Both, //stops us being considered in parent's autosize
+ RelativeSizeAxes = Axes.Both, // stops us being considered in parent's autosize
Anchor = TooltipAnchor.HasFlag(Anchor.x0) ? Anchor.BottomLeft : Anchor.BottomRight,
Origin = TooltipAnchor,
Position = new Vector2(TooltipAnchor.HasFlag(Anchor.x0) ? 5 : -5, 5),
diff --git a/osu.Game/Overlays/VolumeOverlay.cs b/osu.Game/Overlays/VolumeOverlay.cs
index b484921cce..676d2c941a 100644
--- a/osu.Game/Overlays/VolumeOverlay.cs
+++ b/osu.Game/Overlays/VolumeOverlay.cs
@@ -58,7 +58,7 @@ namespace osu.Game.Overlays
{
volumeMeterEffect = new VolumeMeter("EFFECTS", 125, colours.BlueDarker)
{
- Margin = new MarginPadding { Top = 100 + MuteButton.HEIGHT } //to counter the mute button and re-center the volume meters
+ Margin = new MarginPadding { Top = 100 + MuteButton.HEIGHT } // to counter the mute button and re-center the volume meters
},
volumeMeterMaster = new VolumeMeter("MASTER", 150, colours.PinkDarker),
volumeMeterMusic = new VolumeMeter("MUSIC", 125, colours.BlueDarker),
diff --git a/osu.Game/Rulesets/Replays/FramedReplayInputHandler.cs b/osu.Game/Rulesets/Replays/FramedReplayInputHandler.cs
index 7e17396fde..55d82c4083 100644
--- a/osu.Game/Rulesets/Replays/FramedReplayInputHandler.cs
+++ b/osu.Game/Rulesets/Replays/FramedReplayInputHandler.cs
@@ -62,7 +62,7 @@ namespace osu.Game.Rulesets.Replays
{
int newFrame = nextFrameIndex;
- //ensure we aren't at an extent.
+ // ensure we aren't at an extent.
if (newFrame == currentFrameIndex) return false;
currentFrameIndex = newFrame;
@@ -99,8 +99,8 @@ namespace osu.Game.Rulesets.Replays
if (frame == null)
return false;
- return IsImportant(frame) && //a button is in a pressed state
- Math.Abs(CurrentTime - NextFrame?.Time ?? 0) <= AllowedImportantTimeSpan; //the next frame is within an allowable time span
+ return IsImportant(frame) && // a button is in a pressed state
+ Math.Abs(CurrentTime - NextFrame?.Time ?? 0) <= AllowedImportantTimeSpan; // the next frame is within an allowable time span
}
}
diff --git a/osu.Game/Rulesets/RulesetStore.cs b/osu.Game/Rulesets/RulesetStore.cs
index 543134cfb4..f302f8700f 100644
--- a/osu.Game/Rulesets/RulesetStore.cs
+++ b/osu.Game/Rulesets/RulesetStore.cs
@@ -81,7 +81,7 @@ namespace osu.Game.Rulesets
var instances = loadedAssemblies.Values.Select(r => (Ruleset)Activator.CreateInstance(r)).ToList();
- //add all legacy rulesets first to ensure they have exclusive choice of primary key.
+ // add all legacy rulesets first to ensure they have exclusive choice of primary key.
foreach (var r in instances.Where(r => r is ILegacyRuleset))
{
if (context.RulesetInfo.SingleOrDefault(dbRuleset => dbRuleset.ID == r.RulesetInfo.ID) == null)
@@ -90,7 +90,7 @@ namespace osu.Game.Rulesets
context.SaveChanges();
- //add any other modes
+ // add any other modes
foreach (var r in instances.Where(r => !(r is ILegacyRuleset)))
{
if (context.RulesetInfo.FirstOrDefault(ri => ri.InstantiationInfo == r.RulesetInfo.InstantiationInfo) == null)
@@ -99,7 +99,7 @@ namespace osu.Game.Rulesets
context.SaveChanges();
- //perform a consistency check
+ // perform a consistency check
foreach (var r in context.RulesetInfo)
{
try
diff --git a/osu.Game/Rulesets/Scoring/HitResult.cs b/osu.Game/Rulesets/Scoring/HitResult.cs
index 7ba88d3df8..b057af2a50 100644
--- a/osu.Game/Rulesets/Scoring/HitResult.cs
+++ b/osu.Game/Rulesets/Scoring/HitResult.cs
@@ -43,5 +43,25 @@ namespace osu.Game.Rulesets.Scoring
///
[Description(@"Perfect")]
Perfect,
+
+ ///
+ /// Indicates small tick miss.
+ ///
+ SmallTickMiss,
+
+ ///
+ /// Indicates a small tick hit.
+ ///
+ SmallTickHit,
+
+ ///
+ /// Indicates a large tick miss.
+ ///
+ LargeTickMiss,
+
+ ///
+ /// Indicates a large tick hit.
+ ///
+ LargeTickHit
}
}
diff --git a/osu.Game/Scoring/Legacy/ScoreInfoExtensions.cs b/osu.Game/Scoring/Legacy/ScoreInfoExtensions.cs
index 66b1acf591..6f73a284a2 100644
--- a/osu.Game/Scoring/Legacy/ScoreInfoExtensions.cs
+++ b/osu.Game/Scoring/Legacy/ScoreInfoExtensions.cs
@@ -12,7 +12,7 @@ namespace osu.Game.Scoring.Legacy
switch (scoreInfo.Ruleset?.ID ?? scoreInfo.RulesetID)
{
case 3:
- return scoreInfo.Statistics[HitResult.Perfect];
+ return getCount(scoreInfo, HitResult.Perfect);
}
return null;
@@ -35,10 +35,10 @@ namespace osu.Game.Scoring.Legacy
case 0:
case 1:
case 3:
- return scoreInfo.Statistics[HitResult.Great];
+ return getCount(scoreInfo, HitResult.Great);
case 2:
- return scoreInfo.Statistics[HitResult.Perfect];
+ return getCount(scoreInfo, HitResult.Perfect);
}
return null;
@@ -65,7 +65,10 @@ namespace osu.Game.Scoring.Legacy
switch (scoreInfo.Ruleset?.ID ?? scoreInfo.RulesetID)
{
case 3:
- return scoreInfo.Statistics[HitResult.Good];
+ return getCount(scoreInfo, HitResult.Good);
+
+ case 2:
+ return getCount(scoreInfo, HitResult.SmallTickMiss);
}
return null;
@@ -78,6 +81,10 @@ namespace osu.Game.Scoring.Legacy
case 3:
scoreInfo.Statistics[HitResult.Good] = value;
break;
+
+ case 2:
+ scoreInfo.Statistics[HitResult.SmallTickMiss] = value;
+ break;
}
}
@@ -87,10 +94,13 @@ namespace osu.Game.Scoring.Legacy
{
case 0:
case 1:
- return scoreInfo.Statistics[HitResult.Good];
+ return getCount(scoreInfo, HitResult.Good);
case 3:
- return scoreInfo.Statistics[HitResult.Ok];
+ return getCount(scoreInfo, HitResult.Ok);
+
+ case 2:
+ return getCount(scoreInfo, HitResult.LargeTickHit);
}
return null;
@@ -108,6 +118,10 @@ namespace osu.Game.Scoring.Legacy
case 3:
scoreInfo.Statistics[HitResult.Ok] = value;
break;
+
+ case 2:
+ scoreInfo.Statistics[HitResult.LargeTickHit] = value;
+ break;
}
}
@@ -117,7 +131,10 @@ namespace osu.Game.Scoring.Legacy
{
case 0:
case 3:
- return scoreInfo.Statistics[HitResult.Meh];
+ return getCount(scoreInfo, HitResult.Meh);
+
+ case 2:
+ return getCount(scoreInfo, HitResult.SmallTickHit);
}
return null;
@@ -131,13 +148,25 @@ namespace osu.Game.Scoring.Legacy
case 3:
scoreInfo.Statistics[HitResult.Meh] = value;
break;
+
+ case 2:
+ scoreInfo.Statistics[HitResult.SmallTickHit] = value;
+ break;
}
}
public static int? GetCountMiss(this ScoreInfo scoreInfo) =>
- scoreInfo.Statistics[HitResult.Miss];
+ getCount(scoreInfo, HitResult.Miss);
public static void SetCountMiss(this ScoreInfo scoreInfo, int value) =>
scoreInfo.Statistics[HitResult.Miss] = value;
+
+ private static int? getCount(ScoreInfo scoreInfo, HitResult result)
+ {
+ if (scoreInfo.Statistics.TryGetValue(result, out var existing))
+ return existing;
+
+ return null;
+ }
}
}
diff --git a/osu.Game/Screens/BackgroundScreen.cs b/osu.Game/Screens/BackgroundScreen.cs
index 5dfaceccf5..0f3615b7a9 100644
--- a/osu.Game/Screens/BackgroundScreen.cs
+++ b/osu.Game/Screens/BackgroundScreen.cs
@@ -30,7 +30,7 @@ namespace osu.Game.Screens
protected override bool OnKeyDown(KeyDownEvent e)
{
- //we don't want to handle escape key.
+ // we don't want to handle escape key.
return false;
}
diff --git a/osu.Game/Screens/Menu/IntroScreen.cs b/osu.Game/Screens/Menu/IntroScreen.cs
index 736202ee52..0d5f3d1142 100644
--- a/osu.Game/Screens/Menu/IntroScreen.cs
+++ b/osu.Game/Screens/Menu/IntroScreen.cs
@@ -101,7 +101,7 @@ namespace osu.Game.Screens.Menu
this.FadeIn(300);
double fadeOutTime = exit_delay;
- //we also handle the exit transition.
+ // we also handle the exit transition.
if (MenuVoice.Value)
seeya.Play();
else
diff --git a/osu.Game/Screens/Menu/LogoVisualisation.cs b/osu.Game/Screens/Menu/LogoVisualisation.cs
index 67537fa9df..0db7f2a2dc 100644
--- a/osu.Game/Screens/Menu/LogoVisualisation.cs
+++ b/osu.Game/Screens/Menu/LogoVisualisation.cs
@@ -162,7 +162,7 @@ namespace osu.Game.Screens.Menu
private IShader shader;
private Texture texture;
- //Assuming the logo is a circle, we don't need a second dimension.
+ // Assuming the logo is a circle, we don't need a second dimension.
private float size;
private Color4 colour;
@@ -209,13 +209,13 @@ namespace osu.Game.Screens.Menu
float rotation = MathUtils.DegreesToRadians(i / (float)bars_per_visualiser * 360 + j * 360 / visualiser_rounds);
float rotationCos = MathF.Cos(rotation);
float rotationSin = MathF.Sin(rotation);
- //taking the cos and sin to the 0..1 range
+ // taking the cos and sin to the 0..1 range
var barPosition = new Vector2(rotationCos / 2 + 0.5f, rotationSin / 2 + 0.5f) * size;
var barSize = new Vector2(size * MathF.Sqrt(2 * (1 - MathF.Cos(MathUtils.DegreesToRadians(360f / bars_per_visualiser)))) / 2f, bar_length * audioData[i]);
- //The distance between the position and the sides of the bar.
+ // The distance between the position and the sides of the bar.
var bottomOffset = new Vector2(-rotationSin * barSize.X / 2, rotationCos * barSize.X / 2);
- //The distance between the bottom side of the bar and the top side.
+ // The distance between the bottom side of the bar and the top side.
var amplitudeOffset = new Vector2(rotationCos * barSize.Y, rotationSin * barSize.Y);
var rectangle = new Quad(
@@ -231,7 +231,7 @@ namespace osu.Game.Screens.Menu
colourInfo,
null,
vertexBatch.AddAction,
- //barSize by itself will make it smooth more in the X axis than in the Y axis, this reverts that.
+ // barSize by itself will make it smooth more in the X axis than in the Y axis, this reverts that.
Vector2.Divide(inflation, barSize.Yx));
}
}
diff --git a/osu.Game/Screens/Menu/MainMenu.cs b/osu.Game/Screens/Menu/MainMenu.cs
index 0589e4d12b..f0da2482d6 100644
--- a/osu.Game/Screens/Menu/MainMenu.cs
+++ b/osu.Game/Screens/Menu/MainMenu.cs
@@ -250,7 +250,7 @@ namespace osu.Game.Screens.Menu
(Background as BackgroundScreenDefault)?.Next();
- //we may have consumed our preloaded instance, so let's make another.
+ // we may have consumed our preloaded instance, so let's make another.
preloadSongSelect();
if (Beatmap.Value.Track != null && music?.IsUserPaused != true)
diff --git a/osu.Game/Screens/OsuScreen.cs b/osu.Game/Screens/OsuScreen.cs
index 2124a66a75..35bb4fa34f 100644
--- a/osu.Game/Screens/OsuScreen.cs
+++ b/osu.Game/Screens/OsuScreen.cs
@@ -23,7 +23,7 @@ namespace osu.Game.Screens
{
///
/// The amount of negative padding that should be applied to game background content which touches both the left and right sides of the screen.
- /// This allows for the game content to be pushed byt he options/notification overlays without causing black areas to appear.
+ /// This allows for the game content to be pushed by the options/notification overlays without causing black areas to appear.
///
public const float HORIZONTAL_OVERFLOW_PADDING = 50;
diff --git a/osu.Game/Screens/Play/BreakOverlay.cs b/osu.Game/Screens/Play/BreakOverlay.cs
index c978f4e96d..36f825b8f6 100644
--- a/osu.Game/Screens/Play/BreakOverlay.cs
+++ b/osu.Game/Screens/Play/BreakOverlay.cs
@@ -119,7 +119,7 @@ namespace osu.Game.Screens.Play
FinishTransforms(true);
Scheduler.CancelDelayedTasks();
- if (breaks == null) return; //we need breaks.
+ if (breaks == null) return; // we need breaks.
foreach (var b in breaks)
{
diff --git a/osu.Game/Screens/Play/BreakTracker.cs b/osu.Game/Screens/Play/BreakTracker.cs
index fcd7ed6b73..51e21656e1 100644
--- a/osu.Game/Screens/Play/BreakTracker.cs
+++ b/osu.Game/Screens/Play/BreakTracker.cs
@@ -2,40 +2,37 @@
// See the LICENCE file in the repository root for full licence text.
using System.Collections.Generic;
+using System.Linq;
using osu.Framework.Bindables;
using osu.Framework.Graphics;
using osu.Game.Beatmaps.Timing;
using osu.Game.Rulesets.Scoring;
+using osu.Game.Utils;
namespace osu.Game.Screens.Play
{
public class BreakTracker : Component
{
private readonly ScoreProcessor scoreProcessor;
-
private readonly double gameplayStartTime;
+ private PeriodTracker breaks;
+
///
/// Whether the gameplay is currently in a break.
///
public IBindable IsBreakTime => isBreakTime;
- protected int CurrentBreakIndex;
-
private readonly BindableBool isBreakTime = new BindableBool();
- private IReadOnlyList breaks;
-
public IReadOnlyList Breaks
{
- get => breaks;
set
{
- breaks = value;
-
- // reset index in case the new breaks list is smaller than last one
isBreakTime.Value = false;
- CurrentBreakIndex = 0;
+
+ breaks = new PeriodTracker(value.Where(b => b.HasEffect)
+ .Select(b => new Period(b.StartTime, b.EndTime - BreakOverlay.BREAK_FADE_DURATION)));
}
}
@@ -49,34 +46,11 @@ namespace osu.Game.Screens.Play
{
base.Update();
- isBreakTime.Value = getCurrentBreak()?.HasEffect == true
- || Clock.CurrentTime < gameplayStartTime
+ var time = Clock.CurrentTime;
+
+ isBreakTime.Value = breaks?.IsInAny(time) == true
+ || time < gameplayStartTime
|| scoreProcessor?.HasCompleted.Value == true;
}
-
- private BreakPeriod getCurrentBreak()
- {
- if (breaks?.Count > 0)
- {
- var time = Clock.CurrentTime;
-
- if (time > breaks[CurrentBreakIndex].EndTime)
- {
- while (time > breaks[CurrentBreakIndex].EndTime && CurrentBreakIndex < breaks.Count - 1)
- CurrentBreakIndex++;
- }
- else
- {
- while (time < breaks[CurrentBreakIndex].StartTime && CurrentBreakIndex > 0)
- CurrentBreakIndex--;
- }
-
- var closest = breaks[CurrentBreakIndex];
-
- return closest.Contains(time) ? closest : null;
- }
-
- return null;
- }
}
}
diff --git a/osu.Game/Screens/Play/HUD/PlayerSettingsOverlay.cs b/osu.Game/Screens/Play/HUD/PlayerSettingsOverlay.cs
index d201b5d30e..fc80983834 100644
--- a/osu.Game/Screens/Play/HUD/PlayerSettingsOverlay.cs
+++ b/osu.Game/Screens/Play/HUD/PlayerSettingsOverlay.cs
@@ -50,7 +50,7 @@ namespace osu.Game.Screens.Play.HUD
protected override void PopIn() => this.FadeIn(fade_duration);
protected override void PopOut() => this.FadeOut(fade_duration);
- //We want to handle keyboard inputs all the time in order to trigger ToggleVisibility() when not visible
+ // We want to handle keyboard inputs all the time in order to trigger ToggleVisibility() when not visible
public override bool PropagateNonPositionalInputSubTree => true;
protected override bool OnKeyDown(KeyDownEvent e)
diff --git a/osu.Game/Screens/Play/KeyCounter.cs b/osu.Game/Screens/Play/KeyCounter.cs
index f4109a63d0..98df73a5e6 100644
--- a/osu.Game/Screens/Play/KeyCounter.cs
+++ b/osu.Game/Screens/Play/KeyCounter.cs
@@ -124,8 +124,8 @@ namespace osu.Game.Screens.Play
}
}
};
- //Set this manually because an element with Alpha=0 won't take it size to AutoSizeContainer,
- //so the size can be changing between buttonSprite and glowSprite.
+ // Set this manually because an element with Alpha=0 won't take it size to AutoSizeContainer,
+ // so the size can be changing between buttonSprite and glowSprite.
Height = buttonSprite.DrawHeight;
Width = buttonSprite.DrawWidth;
}
diff --git a/osu.Game/Screens/Play/PlayerLoader.cs b/osu.Game/Screens/Play/PlayerLoader.cs
index c0d88feda2..93a734589c 100644
--- a/osu.Game/Screens/Play/PlayerLoader.cs
+++ b/osu.Game/Screens/Play/PlayerLoader.cs
@@ -314,8 +314,8 @@ namespace osu.Game.Screens.Play
LoadTask = null;
- //By default, we want to load the player and never be returned to.
- //Note that this may change if the player we load requested a re-run.
+ // By default, we want to load the player and never be returned to.
+ // Note that this may change if the player we load requested a re-run.
ValidForResume = false;
if (player.LoadedBeatmapSuccessfully)
@@ -360,7 +360,7 @@ namespace osu.Game.Screens.Play
{
if (!muteWarningShownOnce.Value)
{
- //Checks if the notification has not been shown yet and also if master volume is muted, track/music volume is muted or if the whole game is muted.
+ // Checks if the notification has not been shown yet and also if master volume is muted, track/music volume is muted or if the whole game is muted.
if (volumeOverlay?.IsMuted.Value == true || audioManager.Volume.Value <= audioManager.Volume.MinValue || audioManager.VolumeTrack.Value <= audioManager.VolumeTrack.MinValue)
{
notificationOverlay?.Post(new MutedNotification());
diff --git a/osu.Game/Screens/Select/BeatmapCarousel.cs b/osu.Game/Screens/Select/BeatmapCarousel.cs
index 5a4a03662a..96b779cd20 100644
--- a/osu.Game/Screens/Select/BeatmapCarousel.cs
+++ b/osu.Game/Screens/Select/BeatmapCarousel.cs
@@ -208,7 +208,7 @@ namespace osu.Game.Screens.Select
// without this, during a large beatmap import it is impossible to navigate the carousel.
applyActiveCriteria(false, alwaysResetScrollPosition: false);
- //check if we can/need to maintain our current selection.
+ // check if we can/need to maintain our current selection.
if (previouslySelectedID != null)
select((CarouselItem)newSet.Beatmaps.FirstOrDefault(b => b.Beatmap.ID == previouslySelectedID) ?? newSet);
diff --git a/osu.Game/Screens/Select/BeatmapDetailAreaTabControl.cs b/osu.Game/Screens/Select/BeatmapDetailAreaTabControl.cs
index f4bf1ab059..63711e3e50 100644
--- a/osu.Game/Screens/Select/BeatmapDetailAreaTabControl.cs
+++ b/osu.Game/Screens/Select/BeatmapDetailAreaTabControl.cs
@@ -26,7 +26,7 @@ namespace osu.Game.Screens.Select
set => tabs.Current = value;
}
- public Action OnFilter; //passed the selected tab and if mods is checked
+ public Action OnFilter; // passed the selected tab and if mods is checked
public IReadOnlyList TabItems
{
diff --git a/osu.Game/Screens/Select/BeatmapDetails.cs b/osu.Game/Screens/Select/BeatmapDetails.cs
index aebb8e9d87..9669a1391c 100644
--- a/osu.Game/Screens/Select/BeatmapDetails.cs
+++ b/osu.Game/Screens/Select/BeatmapDetails.cs
@@ -201,7 +201,7 @@ namespace osu.Game.Screens.Select
Schedule(() =>
{
if (beatmap != requestedBeatmap)
- //the beatmap has been changed since we started the lookup.
+ // the beatmap has been changed since we started the lookup.
return;
var b = res.ToBeatmap(rulesets);
@@ -222,7 +222,7 @@ namespace osu.Game.Screens.Select
Schedule(() =>
{
if (beatmap != requestedBeatmap)
- //the beatmap has been changed since we started the lookup.
+ // the beatmap has been changed since we started the lookup.
return;
updateMetrics();
diff --git a/osu.Game/Screens/Select/Carousel/CarouselBeatmap.cs b/osu.Game/Screens/Select/Carousel/CarouselBeatmap.cs
index 6d760df065..ed54c158db 100644
--- a/osu.Game/Screens/Select/Carousel/CarouselBeatmap.cs
+++ b/osu.Game/Screens/Select/Carousel/CarouselBeatmap.cs
@@ -2,7 +2,6 @@
// See the LICENCE file in the repository root for full licence text.
using System;
-using System.Collections.Generic;
using System.Linq;
using osu.Game.Beatmaps;
using osu.Game.Screens.Select.Filter;
@@ -55,10 +54,7 @@ namespace osu.Game.Screens.Select.Carousel
if (match)
{
- var terms = new List();
-
- terms.AddRange(Beatmap.Metadata.SearchableTerms);
- terms.Add(Beatmap.Version);
+ var terms = Beatmap.SearchableTerms;
foreach (var criteriaTerm in criteria.SearchTerms)
match &= terms.Any(term => term.IndexOf(criteriaTerm, StringComparison.InvariantCultureIgnoreCase) >= 0);
diff --git a/osu.Game/Screens/Select/Details/AdvancedStats.cs b/osu.Game/Screens/Select/Details/AdvancedStats.cs
index af0d36ea9a..02822ea608 100644
--- a/osu.Game/Screens/Select/Details/AdvancedStats.cs
+++ b/osu.Game/Screens/Select/Details/AdvancedStats.cs
@@ -52,7 +52,7 @@ namespace osu.Game.Screens.Select.Details
AutoSizeAxes = Axes.Y,
Children = new[]
{
- FirstValue = new StatisticRow(), //circle size/key amount
+ FirstValue = new StatisticRow(), // circle size/key amount
HpDrain = new StatisticRow { Title = "HP Drain" },
Accuracy = new StatisticRow { Title = "Accuracy" },
ApproachRate = new StatisticRow { Title = "Approach Rate" },
diff --git a/osu.Game/Screens/Select/PlaySongSelect.cs b/osu.Game/Screens/Select/PlaySongSelect.cs
index 179aab54a3..21ddc5685d 100644
--- a/osu.Game/Screens/Select/PlaySongSelect.cs
+++ b/osu.Game/Screens/Select/PlaySongSelect.cs
@@ -7,6 +7,7 @@ using osu.Framework.Graphics.Sprites;
using osu.Framework.Input.Events;
using osu.Framework.Screens;
using osu.Game.Graphics;
+using osu.Game.Scoring;
using osu.Game.Screens.Play;
using osu.Game.Screens.Ranking;
using osu.Game.Users;
@@ -32,9 +33,12 @@ namespace osu.Game.Screens.Select
Edit();
}, Key.Number4);
- ((PlayBeatmapDetailArea)BeatmapDetails).Leaderboard.ScoreSelected += score => this.Push(new ResultsScreen(score));
+ ((PlayBeatmapDetailArea)BeatmapDetails).Leaderboard.ScoreSelected += PresentScore;
}
+ protected void PresentScore(ScoreInfo score) =>
+ FinaliseSelection(score.Beatmap, score.Ruleset, () => this.Push(new ResultsScreen(score)));
+
protected override BeatmapDetailArea CreateBeatmapDetailArea() => new PlayBeatmapDetailArea();
public override void OnResuming(IScreen last)
diff --git a/osu.Game/Screens/Select/SongSelect.cs b/osu.Game/Screens/Select/SongSelect.cs
index a7e27c27ba..d613b0ae8d 100644
--- a/osu.Game/Screens/Select/SongSelect.cs
+++ b/osu.Game/Screens/Select/SongSelect.cs
@@ -342,13 +342,17 @@ namespace osu.Game.Screens.Select
/// Call to make a selection and perform the default action for this SongSelect.
///
/// An optional beatmap to override the current carousel selection.
- /// Whether to trigger .
- public void FinaliseSelection(BeatmapInfo beatmap = null, bool performStartAction = true)
+ /// An optional ruleset to override the current carousel selection.
+ /// An optional custom action to perform instead of .
+ public void FinaliseSelection(BeatmapInfo beatmap = null, RulesetInfo ruleset = null, Action customStartAction = null)
{
// This is very important as we have not yet bound to screen-level bindables before the carousel load is completed.
if (!Carousel.BeatmapSetsLoaded)
return;
+ if (ruleset != null)
+ Ruleset.Value = ruleset;
+
transferRulesetValue();
// while transferRulesetValue will flush, it only does so if the ruleset changes.
@@ -369,7 +373,12 @@ namespace osu.Game.Screens.Select
selectionChangedDebounce = null;
}
- if (performStartAction && OnStart())
+ if (customStartAction != null)
+ {
+ customStartAction();
+ Carousel.AllowSelection = false;
+ }
+ else if (OnStart())
Carousel.AllowSelection = false;
}
@@ -798,7 +807,7 @@ namespace osu.Game.Screens.Select
Masking = true;
Anchor = Anchor.Centre;
Origin = Anchor.Centre;
- Width = panel_overflow; //avoid horizontal masking so the panels don't clip when screen stack is pushed.
+ Width = panel_overflow; // avoid horizontal masking so the panels don't clip when screen stack is pushed.
InternalChild = Content = new Container
{
RelativeSizeAxes = Axes.Both,
diff --git a/osu.Game/Utils/PeriodTracker.cs b/osu.Game/Utils/PeriodTracker.cs
new file mode 100644
index 0000000000..ba77702247
--- /dev/null
+++ b/osu.Game/Utils/PeriodTracker.cs
@@ -0,0 +1,69 @@
+// Copyright (c) ppy Pty Ltd . Licensed under the MIT Licence.
+// See the LICENCE file in the repository root for full licence text.
+
+using System;
+using System.Collections.Generic;
+using System.Linq;
+
+namespace osu.Game.Utils
+{
+ ///
+ /// Represents a tracking component used for whether a specific time instant falls into any of the provided periods.
+ ///
+ public class PeriodTracker
+ {
+ private readonly List periods;
+ private int nearestIndex;
+
+ public PeriodTracker(IEnumerable periods)
+ {
+ this.periods = periods.OrderBy(period => period.Start).ToList();
+ }
+
+ ///
+ /// Whether the provided time is in any of the added periods.
+ ///
+ /// The time value to check.
+ public bool IsInAny(double time)
+ {
+ if (periods.Count == 0)
+ return false;
+
+ if (time > periods[nearestIndex].End)
+ {
+ while (time > periods[nearestIndex].End && nearestIndex < periods.Count - 1)
+ nearestIndex++;
+ }
+ else
+ {
+ while (time < periods[nearestIndex].Start && nearestIndex > 0)
+ nearestIndex--;
+ }
+
+ var nearest = periods[nearestIndex];
+ return time >= nearest.Start && time <= nearest.End;
+ }
+ }
+
+ public readonly struct Period
+ {
+ ///
+ /// The start time of this period.
+ ///
+ public readonly double Start;
+
+ ///
+ /// The end time of this period.
+ ///
+ public readonly double End;
+
+ public Period(double start, double end)
+ {
+ if (start >= end)
+ throw new ArgumentException($"Invalid period provided, {nameof(start)} must be less than {nameof(end)}");
+
+ Start = start;
+ End = end;
+ }
+ }
+}
diff --git a/osu.Game/osu.Game.csproj b/osu.Game/osu.Game.csproj
index acb7fe5fbe..5ccfaaac9e 100644
--- a/osu.Game/osu.Game.csproj
+++ b/osu.Game/osu.Game.csproj
@@ -18,12 +18,13 @@
+
-
+
-
+
diff --git a/osu.iOS.props b/osu.iOS.props
index 6662e57dcd..dc83d937f7 100644
--- a/osu.iOS.props
+++ b/osu.iOS.props
@@ -70,17 +70,17 @@
-
+
-
+
-
+
diff --git a/osu.Game.Rulesets.Catch/Difficulty/CatchPerformanceCalculator.cs b/osu.Game.Rulesets.Catch/Difficulty/CatchPerformanceCalculator.cs
index a6283eb7c4..e7ce680365 100644
--- a/osu.Game.Rulesets.Catch/Difficulty/CatchPerformanceCalculator.cs
+++ b/osu.Game.Rulesets.Catch/Difficulty/CatchPerformanceCalculator.cs
@@ -52,8 +52,8 @@ namespace osu.Game.Rulesets.Catch.Difficulty
// Longer maps are worth more
double lengthBonus =
- 0.95 + 0.4 * Math.Min(1.0, numTotalHits / 3000.0) +
- (numTotalHits > 3000 ? Math.Log10(numTotalHits / 3000.0) * 0.5 : 0.0);
+ 0.95f + 0.3f * Math.Min(1.0f, numTotalHits / 2500.0f) +
+ (numTotalHits > 2500 ? (float)Math.Log10(numTotalHits / 2500.0f) * 0.475f : 0.0f);
// Longer maps are worth more
value *= lengthBonus;
@@ -63,19 +63,28 @@ namespace osu.Game.Rulesets.Catch.Difficulty
// Combo scaling
if (Attributes.MaxCombo > 0)
- value *= Math.Min(Math.Pow(Attributes.MaxCombo, 0.8) / Math.Pow(Attributes.MaxCombo, 0.8), 1.0);
+ value *= Math.Min(Math.Pow(Score.MaxCombo, 0.8) / Math.Pow(Attributes.MaxCombo, 0.8), 1.0);
- double approachRateFactor = 1.0;
- if (Attributes.ApproachRate > 9.0)
- approachRateFactor += 0.1 * (Attributes.ApproachRate - 9.0); // 10% for each AR above 9
- else if (Attributes.ApproachRate < 8.0)
- approachRateFactor += 0.025 * (8.0 - Attributes.ApproachRate); // 2.5% for each AR below 8
+ float approachRate = (float)Attributes.ApproachRate;
+ float approachRateFactor = 1.0f;
+ if (approachRate > 9.0f)
+ approachRateFactor += 0.1f * (approachRate - 9.0f); // 10% for each AR above 9
+ if (approachRate > 10.0f)
+ approachRateFactor += 0.1f * (approachRate - 10.0f); // Additional 10% at AR 11, 30% total
+ else if (approachRate < 8.0f)
+ approachRateFactor += 0.025f * (8.0f - approachRate); // 2.5% for each AR below 8
value *= approachRateFactor;
if (mods.Any(m => m is ModHidden))
- // Hiddens gives nothing on max approach rate, and more the lower it is
+ {
value *= 1.05 + 0.075 * (10.0 - Math.Min(10.0, Attributes.ApproachRate)); // 7.5% for each AR below 10
+ // Hiddens gives almost nothing on max approach rate, and more the lower it is
+ if (approachRate <= 10.0f)
+ value *= 1.05f + 0.075f * (10.0f - approachRate); // 7.5% for each AR below 10
+ else if (approachRate > 10.0f)
+ value *= 1.01f + 0.04f * (11.0f - Math.Min(11.0f, approachRate)); // 5% at AR 10, 1% at AR 11
+ }
if (mods.Any(m => m is ModFlashlight))
// Apply length bonus again if flashlight is on simply because it becomes a lot harder on longer maps.
diff --git a/osu.Game.Rulesets.Catch/Difficulty/Preprocessing/CatchDifficultyHitObject.cs b/osu.Game.Rulesets.Catch/Difficulty/Preprocessing/CatchDifficultyHitObject.cs
index 24e526ed19..360af1a8c9 100644
--- a/osu.Game.Rulesets.Catch/Difficulty/Preprocessing/CatchDifficultyHitObject.cs
+++ b/osu.Game.Rulesets.Catch/Difficulty/Preprocessing/CatchDifficultyHitObject.cs
@@ -21,10 +21,12 @@ namespace osu.Game.Rulesets.Catch.Difficulty.Preprocessing
public readonly float LastNormalizedPosition;
///
/// Milliseconds elapsed since the start time of the previous , with a minimum of 25ms.
///
public readonly double StrainTime;
+ public readonly double ClockRate;
+
public CatchDifficultyHitObject(HitObject hitObject, HitObject lastObject, double clockRate, float halfCatcherWidth)
: base(hitObject, lastObject, clockRate)
{
@@ -34,8 +36,9 @@ namespace osu.Game.Rulesets.Catch.Difficulty.Preprocessing
NormalizedPosition = BaseObject.X * CatchPlayfield.BASE_WIDTH * scalingFactor;
LastNormalizedPosition = LastObject.X * CatchPlayfield.BASE_WIDTH * scalingFactor;
- // Every strain interval is hard capped at the equivalent of 600 BPM streaming speed as a safety measure
- StrainTime = Math.Max(25, DeltaTime);
+ // Every strain interval is hard capped at the equivalent of 375 BPM streaming speed as a safety measure
+ StrainTime = Math.Max(40, DeltaTime);
+ ClockRate = clockRate;
}
}
}
@@ -21,10 +21,12 @@ namespace osu.Game.Rulesets.Catch.Difficulty.Preprocessing
public readonly float LastNormalizedPosition;
///
- /// Milliseconds elapsed since the start time of the previous , with a minimum of 25ms.
+ /// Milliseconds elapsed since the start time of the previous , with a minimum of 40ms.
///
public readonly double StrainTime;